/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/AddSmProtectResourceCommand.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_HOST;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_TYPE;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmBackupPolicyVerificationSettings;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesResponse;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.snapcenter.cli.command.mapper.CommandMapperInterface;
import com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupScheduleParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupVerificationParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupDataSetParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.ResourceProtectionGroupHelper;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.addsmprotectresource.command.description")
public class AddSmProtectResourceCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMPROTECTRESOURCE_COMMAND_NAME);

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.addsmprotectresource.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-Description" }, descriptionKey = "cli.addsmprotectresource.command.description.option.description")
    private String description;

    @ParametersDelegate
    private BackupResourceGroupParameters backupResourceGroupParameters = new BackupResourceGroupParameters();

    @ParametersDelegate
    private SMOBackupDataSetParameters smoBackupDataSetParameters =  new SMOBackupDataSetParameters();

    @ParametersDelegate
    private VerificationResourceGroupParameters verificationResourceGroupParameters = new VerificationResourceGroupParameters();

    @Parameter(names = { "-Resource" }, descriptionKey = "cli.addsmprotectresource.command.resources.option.description", converter = CustomStringToHostResourcesMapConverter.class, splitter = NoParameterSplitter.class, required = true)
    private Map<String, String> resource;

    @Parameter(names = { "-Schedules" }, descriptionKey = "cli.addsmprotectresource.command.schedules.option.description", converter = CustomStringToSchedulesMapConverter.class, splitter = NoParameterSplitter.class)
    private List<Map<String, String>> schedules = new ArrayList<Map<String, String>>();

    private ExtendedGenericLogger logger;

    // The resource group name will be auto generated.
    private String resourceGroupName;

    private List<SmPolicy> resourceGroupPolicyList;

    private Map<String, List<SmSchedulerType>> policyVsScheduleTypeMap;

    private List<ResourceGroupScheduleParameters> scheduleParametersList;

    private ProgressBar progressBar;

    private CommandMapperInterface commandMapper;

    private boolean isInternal;

    private AddSmResourceGroupCommand addSmResourceGroupCommand;

    private String smObjectGuid;

    public AddSmProtectResourceCommand() {
        isInternal = true;
        addSmResourceGroupCommand = new AddSmResourceGroupCommand();
        policyVsScheduleTypeMap = new HashMap<String, List<SmSchedulerType>>();
        scheduleParametersList = new ArrayList<ResourceGroupScheduleParameters>();
        logger = LoggerFactory
                .getExtendedClassAwareLogger(AddSmProtectResourceCommand.class);
    }

    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        super.initialize();

        /*
         * Process Command arguments to perform validation, setting defaults
         * etc..
         */
        processCommandArguments();

        // Delegate the validation and execution to AddSmResourceGroupCommand,
        // since it is doing the same steps
        addSmResourceGroupCommand.setPluginCode(pluginCode);
        addSmResourceGroupCommand.setResourceGroupName(resourceGroupName);
        addSmResourceGroupCommand.setDescription(description);
        addSmResourceGroupCommand
                .setBackupResourceGroupParameters(backupResourceGroupParameters);
        addSmResourceGroupCommand
                .setSmoBackupDataSetParameters(smoBackupDataSetParameters);
        addSmResourceGroupCommand.setResources(Collections
                .singletonList(resource));
        addSmResourceGroupCommand.setSchedules(schedules);
        addSmResourceGroupCommand.setIsInternal(isInternal);
        addSmResourceGroupCommand.setSnapCenterServerServiceHelper(super
                .getSnapCenterServerServiceHelper());
        addSmResourceGroupCommand
                .setVerificationResourceGroupParameters(verificationResourceGroupParameters);

        addSmResourceGroupCommand.processCommandArguments();

        // Command to request mapping
        SmRequest smRequest = (SmRequest) addSmResourceGroupCommand
                .getCommandMapper().map(addSmResourceGroupCommand);

        SmResponse smResponse = new SmResponse();

        try {
            // Start Progress Bar
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_CONTACTING_SNAPCENTER_SERVER));
            // Invoke Server rest end point
            SMSRestInvocationContext<?, ?> commandRestInvocationContext = getCommandSpecificSMSRestInvocationContext(smRequest);
            smResponse = (SmResponse) super.getSnapCenterServerServiceHelper()
                    .makeRestCallToSnapCenterServer(
                            commandRestInvocationContext);
        } catch (CLIException e) {
            throw e;
        } finally {
            // Stop Progress Bar
            ProgressBarUtil.stopProgressBar(progressBar);
        }

        // Tasks like printing response etc.. are done here.
        super.processCommandResponse(smResponse);

    }

    @Override
    protected void processCommandArguments() throws CLIException {

        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                super.getSnapCenterServerServiceHelper());

        switch (pluginCode) {
            case SCO:
                break;
            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }

        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_VALIDATING_RESOURCE_ALREADY_PROTECTED));

            String resourceName = resource.get(RESOURCE_NAMES);
            String hostName = resource.get(RESOURCE_HOST);
            String resourceType = resource.get(RESOURCE_TYPE);

            if (pluginCode.equals(PluginCode.SCO)) {
                if (resourceType == null
                        || !resourceType.equalsIgnoreCase(ORACLE_DATABASE)) {
                    throw new IllegalArgumentException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_FOR_SMO_RESOURCE_TYPE_SHOULD_BE_ORACLE_DATABASE,
                                            resourceType));

                }
            }

            SmDiscoverResourcesResponse smDiscoverResourcesResponse = smsCommandRequestValidationHelper
                    .getDiscoverResourceDetailsFromServer(
                            SmPluginCode.fromValue(pluginCode.name()),
                            hostName, true);
            // Since only one resource pass the first list element
            smObjectGuid = ResourceProtectionGroupHelper
                    .getSmObjectGuidFromResourceName(
                            smDiscoverResourcesResponse, resourceName);

            // Verify whether resource is already protected by contacting CSM
            // server. If protected throw error.
            SmGetProtectionGroupsResponse smGetProtectionGroupsResponse = smsCommandRequestValidationHelper
                    .getInternalProtectionGroupDetailsFromServer(smObjectGuid);
            if (ResourceProtectionGroupHelper
                    .isResourceAlreadyProtected(smGetProtectionGroupsResponse)) {

                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_SMPROTECTRESOURCE_RESOURCE_ALREADY_PROTECTED,
                                        resourceName));

            }

			SmGetSMTPServerResponse smGetSMTPServerResponse = smsCommandRequestValidationHelper
					.getSmSMTPServerDetailsFromServer();

			SmsCommandArgumentsValidationUtil
					.validateAndSetNotificationParameters(
							backupResourceGroupParameters
									.getNotificationParameters(),
							smGetSMTPServerResponse,COMMANDNAME);

        } catch (Exception exception) {
            throw exception;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }

        /**
         * Polices are mandatory for creating internal resource group
         */
        if (CLIUtils.isNullOrEmptyCollection(backupResourceGroupParameters
                .getPolicies())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SMPROTECTRESOURCE_NO_POLICIES_SPECIFIED));
        }

        /**
         * Set internal resource group name as format
         * "SC_IRG_<host/cluster>_<db name>"
         */
        String[] smObjectGuidSplit = StringUtils.split(smObjectGuid, "\\");
        resourceGroupName = String.format("SC_IRG_%s_%s",
                StringUtils.replace(smObjectGuidSplit[0], ".", "_"),
                smObjectGuidSplit[1]);
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        return addSmResourceGroupCommand
                .getCommandSpecificSMSRestInvocationContext(cmdRequest);
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public BackupResourceGroupParameters getBackupResourceGroupParameters() {
        return backupResourceGroupParameters;
    }

    public void setBackupResourceGroupParameters(
            BackupResourceGroupParameters backupResourceGroupParameters) {
        this.backupResourceGroupParameters = backupResourceGroupParameters;
    }

    public Map<String, String> getResource() {
        return resource;
    }

    public void setResource(Map<String, String> resource) {
        this.resource = resource;
    }

    public List<Map<String, String>> getSchedules() {
        return schedules;
    }

    public void setSchedules(List<Map<String, String>> schedules) {
        this.schedules = schedules;
    }

    public String getResourceGroupName() {
        return resourceGroupName;
    }

    public void setResourceGroupName(String resourceGroupName) {
        this.resourceGroupName = resourceGroupName;
    }

    public List<ResourceGroupScheduleParameters> getScheduleParametersList() {
        return scheduleParametersList;
    }

    public VerificationResourceGroupParameters getVerificationResourceGroupParameters() {
        return verificationResourceGroupParameters;
    }

    public void setVerificationResourceGroupParameters(
            VerificationResourceGroupParameters verificationResourceGroupParameters) {
        this.verificationResourceGroupParameters = verificationResourceGroupParameters;
    }

    public void setScheduleParametersList(
            List<ResourceGroupScheduleParameters> scheduleList) {
        this.scheduleParametersList = scheduleList;
    }

    public List<SmPolicy> getResourceGroupPolicyList() {
        return resourceGroupPolicyList;
    }

    public void setResourceGroupPolicyList(
            List<SmPolicy> resourceGroupPolicyList) {
        this.resourceGroupPolicyList = resourceGroupPolicyList;
    }

    public void setSmoBackupDataSetParameters(
            SMOBackupDataSetParameters smoBackupDataSetParameters) {
        this.smoBackupDataSetParameters = smoBackupDataSetParameters;
    }

    public SMOBackupDataSetParameters getSmoBackupDataSetParameters() {
        return smoBackupDataSetParameters;
    }

    public boolean getIsInternal() {
        return isInternal;
    }

    public void setIsInternal(boolean isInternal) {
        this.isInternal = isInternal;
    }
}
