/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/CancelSmJobCommand.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmCancelJobRequest;
import com.netapp.smcore.plugin.contracts.SmCancelJobResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.CustomStringToLongConverter;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

/**
 * Cancels a queued job.
 */
@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.cancelsmjob.command.description")
public class CancelSmJobCommand extends AbstractSMSCommand {

	private static ExtendedGenericLogger logger = LoggerFactory
			.getExtendedClassAwareLogger(CancelSmJobCommand.class);
	private static final String COMMANDNAME = MessageBundle
			.localizeMessage(CLIMsgKey.CLI_CANCELSMJOB_COMMAND_NAME);
	/**
	 * The generic String to Long converter validates that a valid numeric value
	 * (an integer or long type) is entered by the user. It throws an exception
	 * if the value entered is a decimal or a character.
	 */
	@Parameter(names = { "-JobId" }, descriptionKey = "cli.cancelsmjob.command.jobId.option.description", required = true, converter = CustomStringToLongConverter.class)
	private long jobId;

	@Override
	public String getCommandName() {

		return COMMANDNAME;
	}

	/**
	 * This is to verify that the Job Id entered by a user is a valid numeric
	 * value. Job Id cannot be a negative number, a decimal or a character. Job
	 * Id should be an integer only. The following function checks and throws an
	 * exception if the Job Id entered is negative.
	 */

	@Override
	protected void processCommandArguments() throws CLIException {

		if (getJobId() < 0) {
			throw new CLIException(
					MessageBundle
							.localizeMessage(CLIMsgKey.CLI_JOBID_CANNOT_BE_NEGATIVE));
		}
		CLIUtils.promptUserToContinue(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_CANCELSMJOB_PROMPT_USER_CANCEL_JOB));

	}

	@Override
	protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
			T cmdRequest) {

		SMSRestInvocationContext<SmCancelJobRequest, SmCancelJobResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();

		smsRestInvocationContext.setRequest((SmCancelJobRequest) cmdRequest);

		smsRestInvocationContext.setResponseClass(SmCancelJobResponse.class);

		smsRestInvocationContext
				.setServiceUrlPath(SMSRestConfiguration.SMS_CANCEL_JOB_SERVICE_PATH);

		return smsRestInvocationContext;
	}

	public void setJobId(long jobId) {
		this.jobId = jobId;
	}

	public long getJobId() {
		return jobId;
	}

}
