
/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmConfigSettingsCommand#1 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmAppSettingsParam;
import com.netapp.smcore.plugin.contracts.ConfigSettingsType;
import com.netapp.smcore.plugin.contracts.SmAppSettingsParam;
import com.netapp.smcore.plugin.contracts.SmGetConfigSettingsRequest;
import com.netapp.smcore.plugin.contracts.SmGetConfigSettingsResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;
import org.apache.commons.lang3.StringUtils;


@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmconfigsettings.command.description")
public class GetSmConfigSettingsCommand extends AbstractSMSCommand {


    public static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMCONFIGSETTINGS_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetSmConfigSettingsCommand.class);

    @Parameter(names = { "-PluginCode" } ,descriptionKey = "cli.getsmconfigsettings.command.pluginCode.option.description")
    private PluginCode pluginCode;

    @Parameter(names = { "-HostName" } , descriptionKey = "cli.getsmconfigsettings.command.hostname.option.description")
    private String hostName;

    @Parameter(names = { "-Key" } ,descriptionKey = "cli.getsmconfigsettings.command.key.option.description",required = true)
    private String key;

    @Parameter(names = { "-ConfigSettingsType" } ,descriptionKey = "cli.configsettings.command.configsettingstype.option.decription",required = true)
    ConfigSettingsType configSettingsType;

    @Parameter(names = { "-ShowDescription" } ,descriptionKey = "cli.getsmconfigsettings.command.showDescription.option.description")
    private boolean showDescription;


    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected  void processCommandArguments() throws CLIException {

        //Check if the plugin mentioned is correct

        if(configSettingsType == ConfigSettingsType.PLUGIN && pluginCode == null) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_CONFIGSETTINGS_COMMAND_CONFIGSETTINGSTYPESETTOPLUGIN_PLUGINCODENOTSET_ERROR));
        }
        if(configSettingsType == ConfigSettingsType.PLUGIN) {
            //if Config settings is not specified, then the default type is plugin
            switch(pluginCode) {
                case SCO:
                    break;
                case SCU:
                    break;
                default:
                    throw new CLIException(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                            pluginCode.name()));
            }
        }


        if(StringUtils.isBlank(key)) {
            logger.error("The key specified was null");
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK));
        }
        logger.debug("Getting properties for the " + pluginCode + " plugin.");

        if(getHostName() != null && StringUtils.isBlank(getHostName())){
            throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_CONFIGSETTINGS_COMMAND_HOSTNAME_BLANK_ERROR));
        }

        if ((configSettingsType == ConfigSettingsType.PLUGIN) && getHostName() == null) {
            setHostName(HostInfoUtil.getCanonicalHostName());
            String displayUsingLocalhostDefault = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_GETCONFIGSETTINGS_USING_LOCALHOST_AS_DEFAULT,
                            getHostName());
            CLIUtils.writeToLogAndConsole(displayUsingLocalhostDefault, logger,
                    CLIUtils.LogLevel.INFO);
        }

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(T cmdRequest) {

        SMSRestInvocationContext<SmGetConfigSettingsRequest, SmGetConfigSettingsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();

        smsRestInvocationContext
                .setRequest((SmGetConfigSettingsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetConfigSettingsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_CONFIGSETTINGS_SERVICE_PATH);
        return smsRestInvocationContext;

    }

    public String getHostName() {
        return hostName;
    }

    public void setHostName(String hostName) {
        this.hostName = hostName;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public String getKey() {
        return key;
    }

    public void setKey(String key) {
        this.key = key;
    }

    public ConfigSettingsType getConfigSettingsType() {
        return configSettingsType;
    }

    public void setConfigSettingsType(ConfigSettingsType configSettingsType) {
        this.configSettingsType = configSettingsType;
    }

    public boolean isShowDescription() {
        return showDescription;
    }

    public void setShowDescription(boolean showDescription) {
        this.showDescription = showDescription;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse) throws CLIException {

        SmGetConfigSettingsResponse getConfigSettingsResponse = (SmGetConfigSettingsResponse) smResponse;

        ArrayOfSmAppSettingsParam arrayOfSmAppSettingsParam = getConfigSettingsResponse.getKeyValueDetails();

        if (arrayOfSmAppSettingsParam == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmAppSettingsParam.getSmAppSettingsParam())) {
            return;
        }

        String keyNameHeader = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCONFIGSETTINGS_TABLE_KEY_HEADER);
        String keyValueHeader = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCONFIGSETTINGS_TABLE_VALUE_HEADER);
        String keyDescriptionHeader = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCONFIGSETTINGS_TABLE_DESCRIPTION_HEADER);


        String[] configSettingsTableHeaders = {keyNameHeader,
                keyValueHeader, keyDescriptionHeader};
        TextTableInterface configSettingsTable = new TextTable(configSettingsTableHeaders);

        for (SmAppSettingsParam smAppSettingsParam : arrayOfSmAppSettingsParam.getSmAppSettingsParam()) {
            String columnKeyName = CLIUtils.getStringValue(smAppSettingsParam.getParam());
            String columnKeyValue = CLIUtils.getStringValue(smAppSettingsParam.getType());
            String columnKeyDescription = CLIUtils.getStringValue(smAppSettingsParam.getDetails());
            configSettingsTable.addRow(columnKeyName, columnKeyValue, columnKeyDescription);
        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            if (isShowDescription()) {
                configSettingsTable.displayFormattedTable();
            } else {
                configSettingsTable.displayFormattedTable(keyNameHeader, keyValueHeader);
            }
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmAppSettingsParam);
        }


    }

}
