/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmJobSummaryReportCommand.java#4 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.Date;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmJobSummary;
import com.netapp.smcore.plugin.contracts.SmCreateJobSummaryReportRequest;
import com.netapp.smcore.plugin.contracts.SmCreateJobSummaryReportResponse;
import com.netapp.smcore.plugin.contracts.SmJobSummary;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.CustomStringToDateConverter;
import com.netapp.snapcenter.cli.contracts.GenericConstants;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmjobsummaryreport.command.description")
public class GetSmJobSummaryReportCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_COMMAND_NAME);

    /*
     * TODO: Server enhancement required. Currently if we don't specify any date
     * sever defaults it to current date. Due to server the disadvantage is to
     * get a job report of any job other than current date it will require date
     * of that job also.
     */

    @Parameter(names = { "-Date" }, descriptionKey = "cli.getsmjobsummaryreport.command.date.option.description", converter = CustomStringToDateConverter.class)
    private Date date;

    @Parameter(names = { "-JobId" }, descriptionKey = "cli.getsmjobsummaryreport.command.jobId.option.description")
    private Long jobId;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmCreateJobSummaryReportRequest, SmCreateJobSummaryReportResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmCreateJobSummaryReportRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmCreateJobSummaryReportResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_CREATE_JOBSUMMARYREPORT_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse) throws CLIException {
        SmCreateJobSummaryReportResponse smCreateJobSummaryReportResponse = (SmCreateJobSummaryReportResponse) smResponse;
        ArrayOfSmJobSummary arrayOfSmJobSummary = smCreateJobSummaryReportResponse
                .getJobs();

        if (arrayOfSmJobSummary == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmJobSummary
                        .getSmJobSummary())) {
            return;
        }

        String headerJobId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_TABLE_JOB_ID_HEADER);
        String headerJobName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_TABLE_JOB_NAME_HEADER);
        String headerJobStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_TABLE_JOB_STATUS_HEADER);
        String headerJobType = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_TABLE_JOB_TYPE_HEADER);
        String headerJobError = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_TABLE_JOB_ERROR_HEADER);

        String[] jobSummaryReportTableHeaders = { headerJobId, headerJobName, headerJobStatus,
                                                  headerJobType, headerJobError };
        TextTableInterface jobSummaryReportTable = new TextTable(
                jobSummaryReportTableHeaders);

        //TODO: Need to think of Using Orika for mapping response object

        for (SmJobSummary smJobSummary : arrayOfSmJobSummary.getSmJobSummary()) {

            String columnJobId = smJobSummary.getSmJobId() == null ? ""
                    : smJobSummary.getSmJobId().toString();
            String columnJobName = CLIUtils.getStringValue(smJobSummary
                    .getJobName());
            String columnJobStatus = CLIUtils
                    .getStringValue(smJobSummary.getStatus());
            // Replace Dataset with Resource Group
            if(smJobSummary.getJobType().equals(GenericConstants.DATASET.value())){
                smJobSummary.setJobType(GenericConstants.RESOURCEGROUP.value());
            }
            String columnJobType = CLIUtils.getStringValue(smJobSummary
                    .getJobType());
            String columnJobError = CLIUtils.getStringValue(smJobSummary
                    .getJobError());

            jobSummaryReportTable.addRow(columnJobId, columnJobName, columnJobStatus,
                    columnJobType, columnJobError);
        }
        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            /*
             * If jobId is present in request then user is interested in that
             * particular job and we need to display even job type and error
             * else just display job name and status.
             */
            if (getJobId() != null) {
                jobSummaryReportTable.displayFormattedTable();
            } else {
                jobSummaryReportTable.displayFormattedTable(headerJobId, headerJobName,
                        headerJobStatus);
            }
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmJobSummary);
        }
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public void setJobId(Long jobId) {
        this.jobId = jobId;
    }

    public Date getDate() {
        return date;
    }

    public Long getJobId() {
        return jobId;
    }

}
