/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmResourceGroupCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHostResourceMapping;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPolicy;
import com.netapp.smcore.plugin.contracts.ArrayOfSmProtectionGroup;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsRequest;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmHostResourceMapping;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmProtectionGroup;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmdataset.command.description")
public class GetSmResourceGroupCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_COMMAND_NAME);

    @Parameter(names = { "-ResourceGroupName" }, descriptionKey = "cli.getsmdataset.command.dataSetName.option.description")
    private String datasetName;

    @Parameter(names = { "-ListResources" }, descriptionKey = "cli.getsmdataset.command.listResources.option.description")
    private boolean listResources;

    @Parameter(names = { "-ListPolicies" }, descriptionKey = "cli.getsmdataset.command.listPolicies.option.description")
    private boolean listPolicies;

    // To get datasets this must be true in request
    private boolean displayAll = true;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() {
        /*
         * If dataset is specified then will get all policies and resouces
         * mapped to it
         */
        if (datasetName != null) {
            setListResources(true);
            setListPolicies(true);
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetProtectionGroupsRequest, SmGetProtectionGroupsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetProtectionGroupsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetProtectionGroupsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_PROTECTION_GROUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse) throws CLIException {

        SmGetProtectionGroupsResponse smGetProtectionGroupsResponse = (SmGetProtectionGroupsResponse) smResponse;

        ArrayOfSmProtectionGroup arrayOfSmProtectionGroup = smGetProtectionGroupsResponse
                .getProtectionGroups();
		if (arrayOfSmProtectionGroup == null
				|| CLIUtils.isNullOrEmptyCollection(arrayOfSmProtectionGroup.getSmProtectionGroup())) {
			if (getDatasetName() != null) {
				throw new CLIException(
						MessageBundle.localizeMessage(CLIMsgKey.CLI_RESOURCEGROUP_DOESNOT_EXIST, getDatasetName()));
			} else {
				throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_RESOURCEGROUP_EMPTY));
			}

		}

        String headerDatasetName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_TABLE_DATASET_NAME_HEADER);
        String headerDatasetType = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_TABLE_DATASET_TYPE_HEADER);
        String headerLastBackupStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_TABLE_LASTBACKUPSTATUS_HEADER);
        String headerDatasetDescription = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_TABLE_DATASET_DESCRIPTION_HEADER);
        String headerDatasetPolicies = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_TABLE_DATASET_POLICIES_HEADER);
        String headerDatasetResources = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_TABLE_DATASET_RESOURCES_HEADER);

        String[] datasetTableHeaders = { headerDatasetName, headerDatasetType, headerLastBackupStatus,
                headerDatasetDescription, headerDatasetPolicies, headerDatasetResources };
        TextTableInterface datasetTable = new TextTable(datasetTableHeaders);

        for (SmProtectionGroup smProtectionGroup : arrayOfSmProtectionGroup
                .getSmProtectionGroup()) {

            String columnDatasetName = "";
            String columnDatasetType = "";
            String columnLastBackupStatus = "";
            String columnDatasetDescription = "";
            String columnDatasetPolicies = "";
            String columnDatasetResources = "";

            columnDatasetName = CLIUtils.getStringValue(smProtectionGroup.getName());
            columnDatasetType = CLIUtils.getEnumName(smProtectionGroup
                    .getProtectionGroupType());
            columnLastBackupStatus = CLIUtils.getStringValue(smProtectionGroup
                    .getLastBackupStatus());
            columnDatasetDescription = CLIUtils.getStringValue(smProtectionGroup
                    .getDescription());

            columnDatasetPolicies = getSmPoliciesToString(smProtectionGroup.getPolicies());

            columnDatasetResources = getSmHostResourceMappingToString(smProtectionGroup
                    .getHostResourceMaping());

            datasetTable.addRow(columnDatasetName, columnDatasetType, columnLastBackupStatus,
                    columnDatasetDescription, columnDatasetPolicies, columnDatasetResources);
        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            List<String> displayCols = new ArrayList<>();
            displayCols.add(headerDatasetName);
            if (isListPolicies()) {
                displayCols.add(headerDatasetPolicies);
            }
            if (isListResources()) {
                displayCols.add(headerDatasetResources);
            }
            if (!isListPolicies() && !isListResources()) {
                displayCols.add(headerDatasetType);
                displayCols.add(headerLastBackupStatus);
                displayCols.add(headerDatasetDescription);
            }
            datasetTable.displayFormattedTable(displayCols);
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmProtectionGroup);
        }
    }

    private String getSmPoliciesToString(ArrayOfSmPolicy arrayOfSmPolicy) {
        List<String> policyList = new ArrayList<>();

        if (arrayOfSmPolicy != null
                && !CLIUtils.isNullOrEmptyCollection(arrayOfSmPolicy
                .getSmPolicy())) {
            for (SmPolicy smPolicy : arrayOfSmPolicy.getSmPolicy()) {
                policyList.add(CLIUtils.getStringValue(smPolicy.getName()));
            }
        }

        String policies = StringUtils.substringBetween(policyList.toString(),
                "[", "]");

        return policies;
    }

    private String getSmHostResourceMappingToString(
            ArrayOfSmHostResourceMapping arrayOfSmHostResourceMapping) {

        List<String> hostResourceMapping = new ArrayList<>();

        if (arrayOfSmHostResourceMapping != null
                && !CLIUtils
                .isNullOrEmptyCollection(arrayOfSmHostResourceMapping
                        .getSmHostResourceMapping())) {
            for (SmHostResourceMapping smHostResourceMapping : arrayOfSmHostResourceMapping
                    .getSmHostResourceMapping()) {
                if (smHostResourceMapping.getAppObjects() != null
                        && !CLIUtils
                        .isNullOrEmptyCollection(smHostResourceMapping
                                .getAppObjects().getSmObject())) {
                    for (SmObject smObject : smHostResourceMapping
                            .getAppObjects().getSmObject()) {
                        List<String> resources = new ArrayList<>();
                        resources.add(CLIUtils.getStringValue(smObject
                                .getName()));
                        resources.add(CLIUtils.getStringValue(smObject
                                .getType()));
                        resources.add(CLIUtils.getStringValue(smObject
                                .getHost()));
                        hostResourceMapping.add(resources.toString());
                    }
                }
            }
        }

        String hostResources = hostResourceMapping.toString();

        /**
         * Since hostResources is a list of list while printing we need to
         * remove the outer square brackets.
         */
        hostResources = StringUtils.substring(hostResources, 1,
                hostResources.length() - 1);
        return hostResources;
    }

    public String getDatasetName() {
        return datasetName;
    }

    public void setDatasetName(String datasetName) {
        this.datasetName = datasetName;
    }

    public boolean isListResources() {
        return listResources;
    }

    public void setListResources(boolean listResources) {
        this.listResources = listResources;
    }

    public boolean isListPolicies() {
        return listPolicies;
    }

    public void setListPolicies(boolean listPolicies) {
        this.listPolicies = listPolicies;
    }

    public boolean isDisplayAll() {
        return displayAll;
    }

    public void setDisplayAll(boolean displayAll) {
        this.displayAll = displayAll;
    }

}
