/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmResourcesCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObjectOperationResult;
import com.netapp.smcore.plugin.contracts.ArrayOfSmResourceResult;
import com.netapp.smcore.plugin.contracts.SDHostDevice;
import com.netapp.smcore.plugin.contracts.SDNFSMountPoint;
import com.netapp.smcore.plugin.contracts.SDSANMountPoint;
import com.netapp.smcore.plugin.contracts.SDStorageFootprint;
import com.netapp.smcore.plugin.contracts.SDStorageResource;
import com.netapp.smcore.plugin.contracts.SMResult;
import com.netapp.smcore.plugin.contracts.SmAppFileStorageGroup;
import com.netapp.smcore.plugin.contracts.SmContext;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesRequest;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesResponse;
import com.netapp.smcore.plugin.contracts.SmObjectOperationResult;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmOracleDbType;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResourceResult;
import com.netapp.snapcenter.cli.command.mapper.CustomSMSObjectMapperHelper;
import com.netapp.snapcenter.cli.command.output.formatter.CommandOutputFormatter;
import com.netapp.snapcenter.cli.command.output.formatter.GetSmResourcesCommandOutputFormatter;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.contracts.ResourceType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.OracleDatabaseDetailsUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmresources.command.description")
public class GetSmResourcesCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetSmResourcesCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMRESOURCES_COMMAND_NAME);

    @Parameter(names = { "-HostName" }, descriptionKey = "cli.getsmresources.command.hostName.option.description")
    private String hostName;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmresources.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-ResourceType" }, descriptionKey = "cli.getsmresources.command.resourceType.option.description")
    private ResourceType resourceType;

    /*
     * TODO : Need to decide on exposing this option. For now its hidden
     * 
     * @Parameter(names = { "--resources" }, description = "The plugin " +
     * "resources to be discovered", converter =
     * CustomStringToMapConverter.class, splitter = NoParameterSplitter.class)
     */
    private List<Map<String, String>> resources = new ArrayList<Map<String, String>>();

    @Parameter(names = { "-UseKnownResources" }, descriptionKey = "cli.getsmresources.command.useKnownResources.option.description")
    private boolean useKnownResources;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {


        if ((resourceType != null) && (pluginCode != null)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COMMAND_GETSMRESOURCES_ONLY_ONE_OF_PLUGINCODE_RESOURCETYPE_OPTION_REQUIRED));
        }else if (resourceType == null && pluginCode == null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COMMAND_GETSMRESOURCES_PLUGINCODE_OR_RESOURCETYPE_OPTION_REQUIRED));
        } else if (pluginCode == null) {
            pluginCode = CLIUtils.getPluginTypeFromResourceType(resourceType);
        }

        /**
         * If user has not specified hostname then perform the discovery on
         * local host.
         */
        if (getHostName() == null) {
            setHostName(HostInfoUtil.getCanonicalHostName());
            String displayUsingLocalhostDefault = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_GETSMRESOURCES_USING_LOCALHOST_AS_DEFAULT,
                            getHostName());
            CLIUtils.writeToLogAndConsole(displayUsingLocalhostDefault, logger,
                    LogLevel.INFO);
        }
  
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmDiscoverResourcesRequest, SmDiscoverResourcesResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
     
        smsRestInvocationContext
                .setRequest((SmDiscoverResourcesRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmDiscoverResourcesResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_DISCOVER_RESOURCES_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected CommandOutputFormatter getCommandSpecificOutputFormatter() {
        return new GetSmResourcesCommandOutputFormatter(pluginCode);
    }

    public void setHostName(String hostName) {
        this.hostName = hostName;
    }

    public String getHostName() {
        return hostName;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setResourceType(ResourceType resourceType) {
        this.resourceType = resourceType;
    }

    public ResourceType getResourceType() {
        return resourceType;
    }

    public void setResources(List<Map<String, String>> resources) {
        this.resources = resources;
    }

    public List<Map<String, String>> getResources() {
        return resources;
    }

    public void setUseKnownResources(boolean useKnownResources) {
        this.useKnownResources = useKnownResources;
    }

    public boolean isUseKnownResources() {
        return useKnownResources;
    }
 
}
