/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmResourcesCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmAuth;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmGetRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmGetRunAsAccountResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmrunas.command.description")
public class GetSmRunAsCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMRUNAS_COMMAND_NAME);

    @Parameter(names = { "-Name" }, descriptionKey = "cli.getsmrunas.command.name.option.description")
    private String runAsName;

    @Parameter(names = { "-CredentialType" }, descriptionKey = "cli.getsmrunas.command.authMode.option.description")
    private String authMode;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetRunAsAccountRequest, SmGetRunAsAccountResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetRunAsAccountRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetRunAsAccountResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_RBAC_GET_RUN_AS_ACCOUNT_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse) throws CLIException {

        SmGetRunAsAccountResponse smGetRunAsAccountResponse = (SmGetRunAsAccountResponse) smResponse;

        ArrayOfSmAuth arrayOfSmAuth = smGetRunAsAccountResponse.getAuths();

        if (arrayOfSmAuth == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmAuth.getSmAuth())) {
            return;
        }

        String headerRunAsId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMRUNAS_TABLE_RUNAS_ID_HEADER);
        String headerRunAsName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMRUNAS_TABLE_RUNAS_NAME_HEADER);
        String headerRunAsAuthMode = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMRUNAS_TABLE_RUNAS_AUTHMODE_HEADER);
        String headerRunAsUserName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMRUNAS_TABLE_RUNAS_USERNAME_HEADER);

        String[] runAsTableHeaders = { headerRunAsId,
                                       headerRunAsName,
                                       headerRunAsAuthMode,
                                       headerRunAsUserName };
        TextTableInterface runAsTable = new TextTable(runAsTableHeaders);

        for (SmAuth smAuth : arrayOfSmAuth.getSmAuth()) {
            String columnRunAsId = CLIUtils.getStringValue(Long.toString(smAuth
                    .getId()));
            String columnRunAsName = CLIUtils.getStringValue(smAuth
                    .getRunAsName());
            String columnRunAsAuthMode = CLIUtils.getStringValue(smAuth
                    .getAuthMode().value());
            String columnRunAsUserName = CLIUtils.getStringValue(smAuth
                    .getUserName());
            runAsTable.addRow(columnRunAsId, columnRunAsName,
                    columnRunAsAuthMode, columnRunAsUserName);

        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            runAsTable.displayFormattedTable();
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmAuth);
        }

    }

    public String getRunAsName() {
        return runAsName;
    }

    public void setRunAsName(String runAsName) {
        this.runAsName = runAsName;
    }

    public String getAuthMode() {
        return authMode;
    }

    public void setAuthMode(String authMode) {
        this.authMode = authMode;
    }
}