/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/InvokeSmCloneResourceSplitCommand.java#4 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitEstimateRequest;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitRequest;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitStartRequest;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitStatusRequest;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitStopRequest;
import com.netapp.smcore.plugin.contracts.SmEmailNotificationPreference;
import com.netapp.smcore.plugin.contracts.SmEmailParameters;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmJob;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.command.invokesmcloneresourcesplit.CloneSplitCommandSubType;
import com.netapp.snapcenter.cli.command.invokesmcloneresourcesplit.InvokeSmCloneResourceSplitRestCallFactory;
import com.netapp.snapcenter.cli.command.output.formatter.CommandOutputFormatter;
import com.netapp.snapcenter.cli.command.output.formatter.invokesmresourceclonesplit.OutputFormatterFactory;
import com.netapp.snapcenter.cli.command.parameters.NotificationParameters;
import com.netapp.snapcenter.cli.command.validation.InvokeSmCloneResourceValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.invokesmcloneresourcesplit.command.description")
public class InvokeSmCloneResourceSplitCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMAND_NAME);

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(InvokeSmCloneResourceSplitCommand.class);

    @Parameter(names = { "-CloneName" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.clonename.option.description")
    private String cloneName;

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.appobjectid.option.description")
    private String appObjectId;

    @Parameter(names = { "-Estimate" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.estimate.option.description")
    private boolean estimate;

    @Parameter(names = { "-DoNotEstimate" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.donotestimate.option.description")
    private boolean doNotEstimate;

    @Parameter(names = { "-Start" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.start.option.description")
    private boolean start;

    @Parameter(names = { "-Stop" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.stop.option.description")
    private boolean stop;

    @Parameter(names = { "-Status" }, descriptionKey = "cli.invokesmcloneresourcesplit.command.status.option.description")
    private boolean status;

    @ParametersDelegate
    private NotificationParameters notificationParameters = new NotificationParameters();

    private ProgressBar progressBar;

    CloneSplitCommandSubType cloneSplitCommandSubType;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {

        return InvokeSmCloneResourceSplitRestCallFactory
                .getCommandSpecificSMSRestInvocationContext(
                        cloneSplitCommandSubType, cmdRequest);

    }

    @Override
    protected void processCommandArguments() throws CLIException {

        if (!start && !stop && !status && !estimate) {
            String errorMsg = MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_REQUIRED_OPTION_MISSING_ERROR);
            throw new CLIException(errorMsg);
        }

        if ((start && (stop || estimate || status))
                || (stop && (start || estimate || status))
                || (estimate && (stop || start || status))
                || (status && (stop || estimate || start))) {
            String errorMsg = MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_NOT_SUPPORTED_COMBINATION_ERROR);
            throw new CLIException(errorMsg);
        }

        if (StringUtils.isEmpty(appObjectId) && StringUtils.isEmpty(cloneName)) {
            String errorMsg = MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_APPOBJECTID_OR_CLONENAME_MISSING_ERROR);
            throw new CLIException(errorMsg);
        }

        InvokeSmCloneResourceValidationUtil invokeSmCloneResourceValidationUtil = new InvokeSmCloneResourceValidationUtil(
                super.getSnapCenterServerServiceHelper());
        if (StringUtils.isNotBlank(appObjectId)) {
            invokeSmCloneResourceValidationUtil
                    .getCloneNameFromAppObjectId(appObjectId);

        } else {
            invokeSmCloneResourceValidationUtil.validateCloneName(cloneName);
        }

        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
				super.getSnapCenterServerServiceHelper());
		SmGetSMTPServerResponse smGetSMTPServerResponse = smsCommandRequestValidationHelper
				.getSmSMTPServerDetailsFromServer();
        SmsCommandArgumentsValidationUtil
                .validateAndSetNotificationParameters(notificationParameters,smGetSMTPServerResponse,COMMANDNAME);

    }

    @Override
    public void run() throws CLIException {

        super.initialize();

        /*
         * Process Command arguments to perform validation, setting defaults
         * etc..
         */
        // Start Progress Bar
        progressBar = ProgressBarUtil
                .createAndStartProgressBar(MessageBundle
                        .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_CONTACTING_SNAPCENTER_SERVER));

        // Command to request mapping

        SmResponse smResponse = new SmResponse();

        try {

            processCommandArguments();

            if (setConsoleOutputWidth != null) {
                CLIUtils.setConsoleWidth(setConsoleOutputWidth);
            }

            cloneSplitCommandSubType = getCloneSplitCommandSubType();

            SMSRestInvocationContext<?, ?> smsRestInvocationContext = InvokeSmCloneResourceSplitRestCallFactory
                    .getCommandSpecificSMSRestInvocationContext(
                            cloneSplitCommandSubType,
                            createSmCloneResourceSplitRequest(cloneSplitCommandSubType));

            // Invoke Server rest end point
            smResponse = (SmResponse) super.getSnapCenterServerServiceHelper()
                    .makeRestCallToSnapCenterServer(smsRestInvocationContext);
        } catch (CLIException e) {
            throw e;
        } finally {
            // Stop Progress Bar
            ProgressBarUtil.stopProgressBar(progressBar);
        }

        // Tasks like printing response etc.. are done here.
        super.processCommandResponse(smResponse);

    }

    @Override
    protected CommandOutputFormatter getCommandSpecificOutputFormatter() {

        return OutputFormatterFactory
                .getOutputFormatter(cloneSplitCommandSubType);

    }

    private SmCloneResourceSplitRequest createSmCloneResourceSplitRequest(
            CloneSplitCommandSubType cloneSplitCommandSubType)
            throws CLIException {

        switch (cloneSplitCommandSubType) {
            case ESTIMATE:
                return createSmCloneResourceSplitEstimateRequest();
            case START:
                return createSmCloneResourceSplitStartRequest();
            case STATUS:
                return createSmCloneResourceSplitStatusRequest();
            case STOP:
                return createSmCloneResourceSplitStopRequest();
            default:
                String errorMsg = MessageBundle
                        .localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_UNKNOWN_OPTION_ERROR);
                throw new CLIException(errorMsg);
        }

    }

    /**
     * Create request for Estimate option
     * 
     */
    private SmCloneResourceSplitEstimateRequest createSmCloneResourceSplitEstimateRequest() {
        SmJob smJob = new SmJob();
        smJob.setIsVisible(true);
        SmCloneResourceSplitEstimateRequest smCloneResourceSplitEstimateRequest = new SmCloneResourceSplitEstimateRequest();
        smCloneResourceSplitEstimateRequest.setDisplayAll(true);
        smCloneResourceSplitEstimateRequest.setJob(smJob);
        smCloneResourceSplitEstimateRequest.setAppObjectId(appObjectId);
        smCloneResourceSplitEstimateRequest.setCloneName(cloneName);

        return smCloneResourceSplitEstimateRequest;

    }

    /**
     * Create request for Start option
     * 
     * @throws CLIException
     * 
     */
    private SmCloneResourceSplitStartRequest createSmCloneResourceSplitStartRequest()
            throws CLIException {

        SmJob smJob = new SmJob();
        smJob.setIsVisible(true);
        SmCloneResourceSplitStartRequest smCloneResourceSplitStartRequest = new SmCloneResourceSplitStartRequest();
        smCloneResourceSplitStartRequest.setDisplayAll(true);
        smCloneResourceSplitStartRequest.setJob(smJob);
        smCloneResourceSplitStartRequest.setAppObjectId(appObjectId);
        smCloneResourceSplitStartRequest.setCloneName(cloneName);
        smCloneResourceSplitStartRequest.setDoNotEstimate(doNotEstimate);

        SmEmailParameters smEmailParameters = new SmEmailParameters();
        smEmailParameters.setEmailSubject(notificationParameters
                .getEmailSubject());
        smEmailParameters.setEmailFrom(notificationParameters.getEmailFrom());
        smEmailParameters.setEmailTo(notificationParameters.getEmailTo());
        smEmailParameters
                .setEnableEmail(notificationParameters.isEnableEmail());
		smEmailParameters
				.setEmailNotificationPreference(SmEmailNotificationPreference
						.valueOf(notificationParameters
								.getEmailNotificationPreference().toString()));
        smCloneResourceSplitStartRequest.setEmailParams(smEmailParameters);

        return smCloneResourceSplitStartRequest;

    }

    private SmCloneResourceSplitStopRequest createSmCloneResourceSplitStopRequest() {
        SmCloneResourceSplitStopRequest smCloneResourceSplitStopRequest = new SmCloneResourceSplitStopRequest();
        SmJob smJob = new SmJob();
        smJob.setIsVisible(true);
        smCloneResourceSplitStopRequest.setAppObjectId(appObjectId);
        smCloneResourceSplitStopRequest.setCloneName(cloneName);
        smCloneResourceSplitStopRequest.setJob(smJob);
        smCloneResourceSplitStopRequest.setDisplayAll(true);
        return smCloneResourceSplitStopRequest;
    }

    private SmCloneResourceSplitStatusRequest createSmCloneResourceSplitStatusRequest() {
        SmCloneResourceSplitStatusRequest smCloneResourceSplitStatusRequest = new SmCloneResourceSplitStatusRequest();
        SmJob smJob = new SmJob();
        smJob.setIsVisible(true);
        smCloneResourceSplitStatusRequest.setAppObjectId(appObjectId);
        smCloneResourceSplitStatusRequest.setCloneName(cloneName);
        smCloneResourceSplitStatusRequest.setJob(smJob);
        smCloneResourceSplitStatusRequest.setDisplayAll(true);
        return smCloneResourceSplitStatusRequest;
    }

	private CloneSplitCommandSubType getCloneSplitCommandSubType()
			throws CLIException {
		if (estimate) {
			return CloneSplitCommandSubType.ESTIMATE;
		} else if (start) {
			ProgressBarUtil.stopProgressBar(progressBar);
			CLIUtils.promptUserToContinue(MessageBundle
					.localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_START_OPTION_BACKUPS_DELETE_WARNING_MSG));
			progressBar = ProgressBarUtil
					.createAndStartProgressBar(MessageBundle
							.localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_CONTACTING_SNAPCENTER_SERVER));
			return CloneSplitCommandSubType.START;
		} else if (stop) {
			return CloneSplitCommandSubType.STOP;
		} else if (status) {
			return CloneSplitCommandSubType.STATUS;
		} else {
			String errorMsg = MessageBundle
					.localizeMessage(CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_UNKNOWN_OPTION_ERROR);
			throw new CLIException(errorMsg);
		}

	}

    public String getCloneName() {
        return cloneName;
    }

    public void setCloneName(String cloneName) {
        this.cloneName = cloneName;
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectName) {
        this.appObjectId = appObjectName;
    }

    public boolean getEstimate() {
        return estimate;
    }

    public void setEstimate(boolean value) {
        this.estimate = value;
    }

    public boolean getStart() {
        return start;
    }

    public void setStart(boolean value) {
        this.start = value;
    }

    public boolean getStop() {
        return stop;
    }

    public void setStop(boolean value) {
        this.stop = value;
    }

    public boolean getStatus() {
        return status;
    }

    public void setStatus(boolean value) {
        this.status = value;
    }

    public NotificationParameters getNotificationParameters() {
        return notificationParameters;
    }

    public void setNotificationParameters(
            NotificationParameters notificationParameters) {
        this.notificationParameters = notificationParameters;
    }

}
