/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/ModifySmPolicyCommand.java $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmModifyPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmModifyPolicyResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parameters.BackupPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.CommonBackupCloneVerificationPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.PolicyRetentionSettingsParameters;
import com.netapp.snapcenter.cli.command.parameters.PrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationPrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parser.CustomStringToPolicyRetentionSettingsMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.PolicyCommandParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.contracts.OracleBackupScope;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.contracts.PolicyType;
import com.netapp.snapcenter.cli.contracts.ScheduleType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.modifysmpolicy.command.description")
public class ModifySmPolicyCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_MODIFYSMPOLICY_COMMAND_NAME);

    @Parameter(names = { "-PolicyName" }, descriptionKey = "cli.modifysmpolicy.command.policyName.option.description", required = true)
    private String policyName;

    @Parameter(names = { "-PolicyType" }, descriptionKey = "cli.modifysmpolicy.command.policyType.option.description", required = true)
    private PolicyType policyType;

    /*
     * TODO: Based on app policy type creation of Policy objects differs and
     * need to be implemented once server is ready
     */
    @Parameter(names = { "-PluginPolicyType" }, descriptionKey = "cli.modifysmpolicy.command.pluginPolicyType.option.description")
    private PluginCode pluginPolicyType = PluginCode.SCO;

    @Parameter(names = { "-Description" }, descriptionKey = "cli.modifysmpolicy.command.description.option.description")
    private String description;

    @ParametersDelegate
    private PrePostScriptParameters prePostScriptParameters = new PrePostScriptParameters();

    @ParametersDelegate
    private VerificationPrePostScriptParameters verificationPrePostScriptParameters = new VerificationPrePostScriptParameters();

    @ParametersDelegate
    private CommonBackupCloneVerificationPolicyParameters commonBackupCloneVerificationPolicyParameters = new CommonBackupCloneVerificationPolicyParameters();

    @ParametersDelegate
    private BackupPolicyParameters backupPolicyParameters = new BackupPolicyParameters();

    // Clone policy Not in SCO 1.1 release
    /*
     * @ParametersDelegate private ClonePolicyParameters clonePolicyParameters =
     * new ClonePolicyParameters();
     */
    @ParametersDelegate
    private VerificationPolicyParameters verificationPolicyParameters = new VerificationPolicyParameters();

    @ParametersDelegate
    private SMOBackupPolicyParameters smoBackupPolicyParameters = new SMOBackupPolicyParameters();

    // setting it by default to True
    private boolean modifySchedulerType = true;

    @Parameter(names = { "-RetentionSettings" }, converter = CustomStringToPolicyRetentionSettingsMapConverter.class, splitter = NoParameterSplitter.class)
    private List<Map<String, String>> retentionSettings = new ArrayList<Map<String, String>>();

    private List<PolicyRetentionSettingsParameters> listPolicyRetentionSettingsParameters = new ArrayList<PolicyRetentionSettingsParameters>();

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        switch (pluginPolicyType) {

            case SCO:
                break;

            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginPolicyType.name()));
        }

        /*
         * if (!SmPolicyType.valueOf(policyType.name())
         * .equals(SmPolicyType.BACKUP)) { PolicyCommandParametersValidationUtil
         * .processAndValidateCommonBackupCloneVerificationPolicyParameters(
         * commonBackupCloneVerificationPolicyParameters); }
         */
        SmsCommandArgumentsValidationUtil
                .processAndValidatePrePostScriptParameters(
                        prePostScriptParameters, pluginPolicyType);

        if (!CLIUtils.isEmpty(verificationPrePostScriptParameters
                .getVerificationScriptOptions())
                && (CLIUtils
                        .isEmpty(commonBackupCloneVerificationPolicyParameters
                                .getVerificationScheduleFrequency()))) {
            throw new CLIException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_VERIFICATION_PROVIDED_COMMAND_ARGUMENTS_BUT_SCHEDULE_TYPE_MISSING));

        }

        SmsCommandArgumentsValidationUtil
                .processAndValidateVerificationPrePostScriptParameters(
                        verificationPrePostScriptParameters, pluginPolicyType);

        setScheduleTypeListInPolicyParam(commonBackupCloneVerificationPolicyParameters);

        switch (policyType) {

            case BACKUP:
                if (pluginPolicyType == PluginCode.SCO) {

                    if (!CLIUtils
                            .isEmpty(commonBackupCloneVerificationPolicyParameters
                                    .getListVerificationScheduleTypes())
                            && (smoBackupPolicyParameters
                                    .getOracleBackupScope()
                                    .equals(OracleBackupScope.LOG))) {
                        throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_VERIFICATION_NOT_SUPPORTED_FOR_LOG_ONLY_BACKUP));
                    }
                    PolicyCommandParametersValidationUtil
                            .processAndValidateSMOBackupPolicyParameters(smoBackupPolicyParameters);
                    setListPolicyRetentionSettingsParameters(PolicyCommandParametersValidationUtil
                            .processPolicyRetentionParameters(
                                    retentionSettings,
                                    smoBackupPolicyParameters,
                                    commonBackupCloneVerificationPolicyParameters));
                    PolicyCommandParametersValidationUtil
                            .validateIfScheduleIsValid(
                                    commonBackupCloneVerificationPolicyParameters,
                                    getListPolicyRetentionSettingsParameters(),
                                    policyType.toString());
                }
                break;
            // Clone policy Not in SCO 1.1 release
            /*
             * case CLONE: PolicyCommandParametersValidationUtil
             * .processAndValidateClonePolicyParameters(clonePolicyParameters);
             * break;
             */
            /*
             * case VERIFICATION: PolicyCommandParametersValidationUtil
             * .processAndValidateVerificationPolicyParameters
             * (verificationPolicyParameters); break;
             */

            default:
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_SMPOLICY_CREATION_NOT_AVAILABLE_FOR_POLICY_TYPE,
                                        policyType.name()));

        }

        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                getSnapCenterServerServiceHelper());

        /**
         * TODO: Check with server if the validation is required. If validation
         * is not there then it fails at the server end when user tries to
         * modify schedule* Commenting code as part of review comment. Need
         * confirmation form Krishnan is it handled in server, if not it has to
         * be documented List<String> policies = new ArrayList<String>();
         * policies.add(policyName); //Instead use Collections.singletonList();
         * boolean policyHasSchedule = smsCommandRequestValidationHelper
         * .checkIfPoliciesHaveSchedules(policies, policyType); //policy has a
         * schedule but user did not provide any if (policyHasSchedule &&
         * !(commonBackupCloneVerificationPolicyParameters != null &&
         * commonBackupCloneVerificationPolicyParameters.getScheduleType() !=
         * null )) { throw new CLIException( MessageBundle .localizeMessage(
         * CLIMsgKey
         * .CLI_MODIFYSMPOLICY_COMMAND_SCHEDULE_TYPE_OPTION_NOT_POSSIBLE,
         * policyType.name()));
         * 
         * }
         **/
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmModifyPolicyRequest, SmModifyPolicyResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmModifyPolicyRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmModifyPolicyResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_MODIFY_POLICY_SERVICE_PATH);
        return smsRestInvocationContext;
    }
    
    /*
     * This function converts the list of String to list of ScheduleType
     */
    private static void setScheduleTypeListInPolicyParam(
            CommonBackupCloneVerificationPolicyParameters commonBackupCloneVerificationPolicyParameters)
            throws CLIException {
    	
    	//By default, we add a none schedule to take care of on-demand schedule type. So validating the 
    	//size of list for 2 - NONE and user provided schedule type.
    	if(commonBackupCloneVerificationPolicyParameters.getScheduleType().size() > 2){
    		throw new CLIException(MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_ADDSMPOLICY_MULTIPLESCHEDULES_ERROR));
    	}

        // First remove the duplicates from list and validates with ScheduleType
        CLIUtils.removeDuplicateEntriesFromList(commonBackupCloneVerificationPolicyParameters
                .getScheduleType());
        commonBackupCloneVerificationPolicyParameters
                .setScheduleTypeAsListOfScheduleType(CLIUtils
                        .convertStringListToEnumList(ScheduleType.class,
                                commonBackupCloneVerificationPolicyParameters
                                        .getScheduleType()));

        if (!CLIUtils.isEmpty(commonBackupCloneVerificationPolicyParameters
                .getVerificationScheduleFrequency())) {
            CLIUtils.removeDuplicateEntriesFromList(commonBackupCloneVerificationPolicyParameters
                    .getVerificationScheduleFrequency());
            if(commonBackupCloneVerificationPolicyParameters.getVerificationScheduleFrequency().size() > 1){
            	//Checking for only size 1 because there is no on-demand verification schedule(NONE schedule type).
            	throw new CLIException(MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_ADDSMPOLICY_MULTIPLESCHEDULESFORVERIFICATION_ERROR));
            }
            commonBackupCloneVerificationPolicyParameters
                    .setListVerificationScheduleTypes(CLIUtils
                            .convertStringListToEnumList(ScheduleType.class,
                                    commonBackupCloneVerificationPolicyParameters
                                            .getVerificationScheduleFrequency()));
        }
    }

    public String getPolicyName() {
        return policyName;
    }

    public void setPolicyName(String policyName) {
        this.policyName = policyName;
    }

    public void setPolicyType(PolicyType policyType) {
        this.policyType = policyType;
    }

    public PolicyType getPolicyType() {
        return policyType;
    }

    public void setPluginPolicyType(PluginCode pluginPolicyType) {
        this.pluginPolicyType = pluginPolicyType;
    }

    public PluginCode getPluginPolicyType() {
        return pluginPolicyType;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public BackupPolicyParameters getBackupPolicyParameters() {
        return backupPolicyParameters;
    }

    public void setBackupPolicyParameters(
            BackupPolicyParameters backupPolicyParameters) {
        this.backupPolicyParameters = backupPolicyParameters;
    }

    public SMOBackupPolicyParameters getSmoBackupPolicyParameters() {
        return smoBackupPolicyParameters;
    }

    public void setSmoBackupPolicyParameters(
            SMOBackupPolicyParameters smoBackupPolicyParameters) {
        this.smoBackupPolicyParameters = smoBackupPolicyParameters;
    }

    // Clone policy Not in SCO 1.1 release
    /*
     * public void setClonePolicyParameters( ClonePolicyParameters
     * clonePolicyParameters) { this.clonePolicyParameters =
     * clonePolicyParameters; }
     * 
     * public ClonePolicyParameters getClonePolicyParameters() { return
     * clonePolicyParameters; }
     */
    public void setCommonBackupClonePolicyParameters(
            CommonBackupCloneVerificationPolicyParameters commonBackupCloneVerificationPolicyParameters) {
        this.commonBackupCloneVerificationPolicyParameters = commonBackupCloneVerificationPolicyParameters;
    }

    public CommonBackupCloneVerificationPolicyParameters getCommonBackupClonePolicyParameters() {
        return commonBackupCloneVerificationPolicyParameters;
    }

    public boolean isModifySchedulerType() {
        return modifySchedulerType;
    }

    public void setModifySchedulerType(boolean modifySchedulerType) {
        this.modifySchedulerType = modifySchedulerType;
    }

    public void setVerificationPolicyParameters(
            VerificationPolicyParameters verificationPolicyParameters) {
        this.verificationPolicyParameters = verificationPolicyParameters;
    }

    public VerificationPolicyParameters getVerificationPolicyParameters() {
        return verificationPolicyParameters;
    }

    public void setPrePostScriptParameters(
            PrePostScriptParameters prePostScriptParameters) {
        this.prePostScriptParameters = prePostScriptParameters;
    }

    public PrePostScriptParameters getPrePostScriptParameters() {
        return prePostScriptParameters;
    }

    public VerificationPrePostScriptParameters getVerificationPrePostScriptParameters() {
        return verificationPrePostScriptParameters;
    }

    public void setVerificationPrePostScriptParameters(
            VerificationPrePostScriptParameters verificationPrePostScriptParameters) {
        this.verificationPrePostScriptParameters = verificationPrePostScriptParameters;
    }

    public List<PolicyRetentionSettingsParameters> getListPolicyRetentionSettingsParameters() {
        return listPolicyRetentionSettingsParameters;
    }

    public void setListPolicyRetentionSettingsParameters(
            List<PolicyRetentionSettingsParameters> listPolicyRetentionSettingsParameters) {
        this.listPolicyRetentionSettingsParameters = listPolicyRetentionSettingsParameters;
    }

}
