/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/RefreshSmCloneCommand.java#4 $
 * 
 * Copyright (c) 2018 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_HOST;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_TYPE;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import com.netapp.snapcenter.cli.command.parameters.NotificationParameters;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.ArrayOfSmArchivedDetail;
import com.netapp.smcore.plugin.contracts.ArrayOfSmArchivedLocator;
import com.netapp.smcore.plugin.contracts.SmBackupFilter;
import com.netapp.smcore.plugin.contracts.SmArchivedDetail;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmCreateBackupRequest;
import com.netapp.smcore.plugin.contracts.SmCreateBackupResponse;
import com.netapp.smcore.plugin.contracts.SmGetBackupRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupResponse;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneResponse;
import com.netapp.smcore.plugin.contracts.SmCreateCloneRequest;
import com.netapp.smcore.plugin.contracts.SmCreateCloneResponse;
import com.netapp.smcore.plugin.contracts.SmGetCloneNameResponse;
import com.netapp.smcore.plugin.contracts.SmGetHostResponse;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmGetRolesByUserResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SnapshotLocator;
import com.netapp.smcore.plugin.contracts.ConfigSettingsType;
import com.netapp.smcore.plugin.contracts.SmAppSettingsParam;
import com.netapp.smcore.plugin.contracts.SmGetConfigSettingsRequest;
import com.netapp.smcore.plugin.contracts.SmGetConfigSettingsResponse;
import com.netapp.snapcenter.cli.clone.contracts.SecondaryLocator;
import com.netapp.snapcenter.cli.clone.contracts.SecondaryLocators;
import com.netapp.snapcenter.cli.clone.contracts.OracleCloneSpecification;
import com.netapp.snapcenter.cli.command.parameters.PrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOCloneParameters;
import com.netapp.snapcenter.cli.command.validation.CloneCommandParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.InvokeSmCloneResourceValidationUtil;
import com.netapp.snapcenter.cli.command.validation.OracleCloneSpecificationValidationUtil;
import com.netapp.snapcenter.cli.command.validation.PolicyCommandParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.ResourceProtectionGroupHelper;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.snapcenter.cli.utils.SMSObjectSerializationUtil;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.refreshsmclone.command.description")
public class RefreshSmCloneCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(RefreshSmCloneCommand.class);

    @Parameter(names = { "-CloneToHost" }, descriptionKey = "cli.refreshsmclone.command.cloneToHost.option.description")
    private String cloneToHost;

    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.refreshsmclone.command.backupName.option.description")
    private String backupName;
    
    @Parameter(names = { "-PolicyName" }, descriptionKey = "cli.refreshsmclone.command.policy.option.description")
    private String policyName;
    
    @Parameter(names = { "-ResourceGroupName" }, descriptionKey = "cli.refreshsmclone.command.resourcegroup.option.description")
    private String resourceGroupName;
    
    @Parameter(names = { "-CloneDatabaseSID" }, descriptionKey = "cli.newsmclone.command.cloneDatabaseSid.option.description")
    private String cloneDatabaseSid;
    
    @Parameter(names = { "-SecondaryUpdateWaitTimeout" }, descriptionKey = "cli.refreshsmclone.command.secondaryUpdateWaitTimeout.option.description")
    private long secondaryUpdateWaitTimeout=0;
    
    @Parameter(names = { "-WaitToTriggerClone" }, descriptionKey = "cli.refreshsmclone.command.waitToTriggerClone.option.description")
    private long waitToTriggerClone=1;
    
    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.refreshsmclone.command.waitForCompletion.option.description")
    private boolean waitForCompletion;

    @ParametersDelegate
    SMOCloneParameters smoCloneParameters = new SMOCloneParameters();

    @ParametersDelegate
    private PrePostScriptParameters prePostScriptParameters = new PrePostScriptParameters();

    @ParametersDelegate
    private NotificationParameters notificationParameters = new NotificationParameters();
    
    private PluginCode appPluginCode = PluginCode.SCO;
    
    private String backupId;
    
    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmCreateCloneRequest, SmCreateCloneResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmCreateCloneRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmCreateCloneResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_CREATE_CLONE_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
    	// Validating request..
        if ( !StringUtils.isBlank(cloneToHost)) {
                SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(super.getSnapCenterServerServiceHelper());
                SmGetHostResponse smGetHostResponse = smsCommandRequestValidationHelper.getSmHost(cloneToHost);
                
				if (smGetHostResponse.getHostInfo().getHosts().getSmHost()
						.isEmpty()) {
					String error = MessageBundle.localizeMessage(
							CLIMsgKey.CLI_PARAMETER_SC_OPERATION_INVALID_HOST,
							cloneToHost);
					throw new CLIException(error);
				}

        }
        if (StringUtils.isBlank(policyName)) {
            throw new CLIException(
                    MessageBundle.localizeMessage(CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_INVALID_POLICY_OPTIONS));
        }
        
        PolicyCommandParametersValidationUtil.isValidPolicyName(policyName);
        CloneCommandParametersValidationUtil.validateAndSetSMOCloneParameters(smoCloneParameters);
        // Validation end...
        
        try {
        	CLIUtils.writeToLog("processCommandArguments: Enter", logger,LogLevel.INFO);
        	SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(getSnapCenterServerServiceHelper());
            String cloneSpecfile = smoCloneParameters.getOracleCloneSpecificationFile();
            
            JAXBContext jaxbContext = JAXBContext
                    .newInstance(OracleCloneSpecification.class);
            Unmarshaller unmarshaller = jaxbContext
                    .createUnmarshaller();
            OracleCloneSpecification oracleCloneSpecification = (OracleCloneSpecification) unmarshaller
                    .unmarshal(new File(cloneSpecfile));
            if(oracleCloneSpecification != null && !StringUtils.isBlank(cloneDatabaseSid)) {
            	CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
            			CLIMsgKey.CLI_NEWSMCLONE_COMMAND_SETTING_SPECIFIED_CLONE_DATABASE_SID, 
            			cloneDatabaseSid),
            			logger, LogLevel.INFO);
            	oracleCloneSpecification.setCloneDatabaseSid(cloneDatabaseSid);
            }
            OracleCloneSpecificationValidationUtil
                    .validateCloneSpec(oracleCloneSpecification, COMMANDNAME);

            if (StringUtils.isBlank(getCloneToHost())) {
                setCloneToHost(HostInfoUtil.getCanonicalHostName());
                String usingDefaultHost = MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_NEWSMCLONE_USING_LOCALHOST_AS_DEFAULT,
                        getCloneToHost());
                CLIUtils.writeToLogAndConsole(usingDefaultHost, logger,
                        LogLevel.INFO);
            }
            
            if (StringUtils.isBlank(backupName)){
            	createBackup(smsBackupValidationHelper, oracleCloneSpecification);
            } else {
            	setBackupId(smsBackupValidationHelper
                .fetchBackupIdIfItIsValidOracleDataBackup(backupName,
                		oracleCloneSpecification.getAppobjectid(), false));
            	if (StringUtils.isBlank(getBackupId())) {
            		throw new CLIException(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_INVALID_BACKUP,
                            backupName));
            	}
            }
            CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_FOR_CLONE,
                    backupName), logger,LogLevel.INFO);
            oracleCloneSpecification.setBackupname(backupName);
        	smoCloneParameters.setOracleCloneSpecification(oracleCloneSpecification);
            
            String cloneAppObjectId = getCloneToHost() + '\\' + oracleCloneSpecification.getCloneDatabaseSid();
            SmResponse deleteCloneResponse = deleteClone(cloneAppObjectId);

            if (deleteCloneResponse.getResult() != null && StringUtils.isNotEmpty(deleteCloneResponse.getResult().getMessage())){
            	throw new CLIException(deleteCloneResponse.getResult().getMessage());
            }

        } catch (JAXBException e) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_UNABLE_TO_PARSE_CLONE_SPEC_FILE),
                    e);
        }

        SmsCommandArgumentsValidationUtil
                .processAndValidatePrePostScriptParameters(getPrePostScriptParameters(), getAppPluginCode());

		SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
				super.getSnapCenterServerServiceHelper());
		SmGetSMTPServerResponse smGetSMTPServerResponse = smsCommandRequestValidationHelper
				.getSmSMTPServerDetailsFromServer();

		SmsCommandArgumentsValidationUtil.validateAndSetNotificationParameters(
				notificationParameters, smGetSMTPServerResponse,COMMANDNAME);
		
		if(waitToTriggerClone > 0)
		{
			CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_WAITTOTRIGGERCLONE,
                    waitToTriggerClone), logger,LogLevel.INFO);
			try {
				Thread.sleep(waitToTriggerClone*60000);
            } catch (InterruptedException e) {
            	CLIUtils.writeToLog(e.getLocalizedMessage(), logger,LogLevel.INFO);
            }
		}
		CLIUtils.writeToLog("processCommandArguments: Exit", logger,LogLevel.INFO);

    }
    
    private SmResponse deleteClone(String cloneAppObjectId) throws CLIException {
    	CLIUtils.writeToLog("deleteClone: Enter", logger,LogLevel.INFO);
    	InvokeSmCloneResourceValidationUtil invokeSmCloneResourceValidationUtil = new InvokeSmCloneResourceValidationUtil(
				super.getSnapCenterServerServiceHelper());
    	SmResponse smResponse = new SmResponse();
    	ProgressBar progressBar = new ProgressBar();
        try {
        	
        	SmGetCloneNameResponse smGetCloneNameResponse = invokeSmCloneResourceValidationUtil
    				.getCloneNameFromAppObjectId(cloneAppObjectId);
        	if (StringUtils.isEmpty(smGetCloneNameResponse.getCloneName()))
        		throw new CLIException(MessageBundle
                        .localizeMessage(CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_NOCLONE_TODELETE, cloneAppObjectId));
        	
        	RemoveSmCloneCommand delCloneRequest = new RemoveSmCloneCommand();
        	List<String> cloneNames = new ArrayList<>();
        	cloneNames.add(smGetCloneNameResponse.getCloneName());
        	delCloneRequest.setCloneNames(cloneNames);

        	// Command to request mapping
            SmRequest smRequest = (SmRequest) delCloneRequest.getCommandMapper().map(delCloneRequest);
            
            // Start Progress Bar
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_REFRESHCLONE_DELETE_CLONE) + smGetCloneNameResponse.getCloneName());
            // Invoke Server rest end point
            SMSRestInvocationContext<SmDeleteCloneRequest, SmDeleteCloneResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
            smsRestInvocationContext.setRequest((SmDeleteCloneRequest) smRequest);
            smsRestInvocationContext.setResponseClass(SmDeleteCloneResponse.class);
            smsRestInvocationContext
                    .setServiceUrlPath(SMSRestConfiguration.SMS_DELETE_CLONE_SERVICE_PATH);
            smResponse = (SmResponse) super.getSnapCenterServerServiceHelper()
                    .makeRestCallToSnapCenterServer(
                    		smsRestInvocationContext);

            if (smResponse.getJob() == null || smResponse.getJob().getId() <= 0){
            	CLIUtils.writeToLog("Failed to get job details" + smResponse.getResult().getMessage(), logger,LogLevel.INFO);
            	return smResponse;
            }
            getSnapCenterServerServiceHelper().pollSMSJobStatus(
                    smResponse.getJob().getId());
            
        } catch (CLIException e) {
        	CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_CLONEDELETION_FAILED, e.getMessage()),
        			logger, LogLevel.INFO);
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);
        }
        CLIUtils.writeToLog("deleteClone: Exit", logger,LogLevel.INFO);
        return smResponse;
    }

    private void createBackup(SmsBackupValidationHelper smsBackupValidationHelper, OracleCloneSpecification oracleCloneSpec) throws CLIException {

    	CLIUtils.writeToLog("createBackup: Enter", logger,LogLevel.INFO);
    	ProgressBar progressBar = new ProgressBar();
            // When resource is provided get the protection group name from the server
            try {
            	String smObjectGuid = oracleCloneSpec.getAppobjectid();
            	SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                        getSnapCenterServerServiceHelper());

                if(StringUtils.isEmpty(resourceGroupName)){
                	// Verify whether resource is already protected by contacting
                    // CSM server. If not protected throw error.
                    SmGetProtectionGroupsResponse smGetProtectionGroupsResponse = smsCommandRequestValidationHelper
                            .getInternalProtectionGroupDetailsFromServer(smObjectGuid);
                    if (!ResourceProtectionGroupHelper
                            .isResourceAlreadyProtected(smGetProtectionGroupsResponse)) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_SMPROTECTRESOURCE_RESOURCE_NOT_PROTECTED,
                                                smObjectGuid));
                    }
                	resourceGroupName = ResourceProtectionGroupHelper
                        .getInternalResourceGroupNameFromProtectionGroup(
                                smGetProtectionGroupsResponse, smObjectGuid);
                }
                CLIUtils.writeToLog("Using resource group " + resourceGroupName, logger,LogLevel.INFO);
                
                NewSmBackupCommand createBackupRequest = new NewSmBackupCommand();
                createBackupRequest.setPolicyName(policyName);
                createBackupRequest.setResourceGroup(resourceGroupName);
                createBackupRequest.setProtectoinGroupName(resourceGroupName);
            	// Command to request mapping
                SmRequest smRequest = (SmRequest) createBackupRequest.getCommandMapper().map(createBackupRequest);
                
                SmResponse smResponse = new SmResponse();
                try {
                    // Start Progress Bar
                	CLIUtils.writeToLogAndConsole(MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_REFRESHCLONE_CREATE_BACKUP)+ smObjectGuid, logger, LogLevel.INFO);
                    // Invoke Server rest end point
                    SMSRestInvocationContext<SmCreateBackupRequest, SmCreateBackupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
                    smsRestInvocationContext.setRequest((SmCreateBackupRequest) smRequest);
                    smsRestInvocationContext.setResponseClass(SmCreateBackupResponse.class);
                    smsRestInvocationContext
                            .setServiceUrlPath(SMSRestConfiguration.SMS_CREATE_BACKUP_SERVICE_PATH);
                    smResponse = (SmResponse) super.getSnapCenterServerServiceHelper()
                            .makeRestCallToSnapCenterServer(
                            		smsRestInvocationContext);                    
                } catch (CLIException e) {
                    throw e;
                }
                
                if (smResponse.getJob() == null || smResponse.getJob().getId() <= 0){
                	CLIUtils.writeToLog("Failed to get job details" + smResponse.getResult().getMessage(), logger,LogLevel.INFO);
                	throw new CLIException(smResponse.getResult().getMessage());
                }
                getSnapCenterServerServiceHelper().pollSMSJobStatus(smResponse.getJob().getId());
                getBackupNameFromJob(smsBackupValidationHelper, oracleCloneSpec, smResponse.getJob().getId());

            } catch (Exception exception) {
                throw exception;
            } finally {
                ProgressBarUtil.stopProgressBar(progressBar);
            }
            CLIUtils.writeToLog("createBackup: Exit", logger,LogLevel.INFO);

    }
    
    private void getBackupNameFromJob(SmsBackupValidationHelper smsBackupValidationHelper, OracleCloneSpecification oracleCloneSpec, long jobId) throws CLIException {
    	InvokeSmCloneResourceValidationUtil invokeSmCloneResourceValidationUtil = new InvokeSmCloneResourceValidationUtil(
				super.getSnapCenterServerServiceHelper());
    	CLIUtils.writeToLog("getBackupNameFromJob: Enter", logger,LogLevel.INFO);
        SmGetBackupResponse response = new SmGetBackupResponse();
        ProgressBar progressBar = new ProgressBar();

        try {
            // Start Progress Bar
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_REFRESHCLONE_GET_BACKUP));

            SmGetBackupRequest request = new SmGetBackupRequest();

            SmBackup smBackup = new SmBackup();
            smBackup.setJobId(jobId);
            request.setBackup(smBackup);
            SmObject smObject = new SmObject();
            smObject.setId(oracleCloneSpec.getAppobjectid());
            request.setAppObject(smObject);

            SmBackupFilter smBackupFilter = new SmBackupFilter();
            smBackupFilter.setJobId(jobId);
            request.setBackupFilter(smBackupFilter);

            SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                    .createSMSRestInvocationContext(request,
                            SmGetBackupResponse.class,
                            SMSRestConfiguration.SMS_GET_BACKUPS_SERVICE_PATH);
            response = (SmGetBackupResponse) getSnapCenterServerServiceHelper().makeRestCallToSnapCenterServer(smsRestInvocationContext);

            if (!getSnapCenterServerServiceHelper()
                    .isSnapCenterRestOperationResponseResultSuccess(
                            response.getResult())) {
                throw new CLIException(response.getResult().getMessage());
            }
            SmBackup backup = smsBackupValidationHelper.getFirstBackupFromSmGetBackupResponse(response);
            if (backup != null && StringUtils.isNotBlank(backup.getBackupId())
                    && StringUtils.isNotBlank(backup.getBackupName())
                    && StringUtils.isNotBlank(backup.getBackupType())) {
                if (backup.getBackupType().toLowerCase()
                        .contains(SmsBackupValidationHelper.ORACLE_DATA_BACKUP_TYPE.toLowerCase())) {
                    backupId = backup.getBackupId();
                    backupName = backup.getBackupName();
                    CLIUtils.writeToLog("Data backup id:" + backupId, logger,LogLevel.INFO);
                    CLIUtils.writeToLog("Data backup name:" + backupName, logger,LogLevel.INFO);
                }
            }
            if (StringUtils.isEmpty(backupId) || StringUtils.isEmpty(backupName))
            	throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_BACKUP_NOTFOUND_ONPRIMARY));
            
            if (!StringUtils.isEmpty(backupId) && 
            		oracleCloneSpec.getStorageSpecification().getStorageMapping().getSecondaryConfigurationDatafiles() != null &&
            		oracleCloneSpec.getStorageSpecification().getStorageMapping().getSecondaryConfigurationArchivelogs() != null){
            	pollIfBackupReachedSecondary(oracleCloneSpec, smsBackupValidationHelper);
            	for(SmBackup logbackup: response.getBackups().getSmBackup()){
            		if (logbackup != null && StringUtils.isNotBlank(logbackup.getBackupId())
                            && StringUtils.isNotBlank(logbackup.getBackupName())
                            && StringUtils.isNotBlank(logbackup.getBackupType())) {
                        if (logbackup.getBackupType().toLowerCase()
                                .contains(SmsBackupValidationHelper.ORACLE_LOG_BACKUP_TYPE.toLowerCase())) {
                        	CLIUtils.writeToLog("Log backup " + logbackup.getBackupName(), logger,LogLevel.INFO);
                            pollIfLogBackupReachedSecondary(oracleCloneSpec, smsBackupValidationHelper, logbackup.getBackupId(), logbackup.getBackupName());
                            break;
                        }
                    }
            	}
            }
            
        } catch (CLIException e) {
            throw e;
        } finally {
            // Stop Progress Bar
            ProgressBarUtil.stopProgressBar(progressBar);
        }
        CLIUtils.writeToLog("getBackupNameFromJob: Exit", logger,LogLevel.INFO);
    }
    
    private void pollIfBackupReachedSecondary(OracleCloneSpecification oracleCloneSpec, SmsBackupValidationHelper smsBackupValidationHelper) throws CLIException {

    	CLIUtils.writeToLog("pollIfBackupReachedSecondary: Enter", logger,LogLevel.INFO);
    	ProgressBar progressBar = new ProgressBar();
        try {
	        int retryCount = 1;
	        int waitTime = 60;
	        long maxRetryCount = 15;
	        boolean found = false;
	        
	        if (secondaryUpdateWaitTimeout > 0) {
	        	maxRetryCount = secondaryUpdateWaitTimeout;
	        	CLIUtils.writeToLog("Setting max retry count to " + maxRetryCount, logger, LogLevel.INFO);
	        }	        
	        
	        while(retryCount <= maxRetryCount) {
	        	try{
			    	SmGetArchivedDetailsResponse smGetArchivedDetailsResponse = smsBackupValidationHelper
			                .getSecondaryBackupDetails(backupId, oracleCloneSpec.getAppobjectid(), appPluginCode, true);
			    	
			    	ArrayOfSmArchivedDetail arrayOfSmArchivedDetail = smGetArchivedDetailsResponse.getArchivedDetails();
	
			    	if (arrayOfSmArchivedDetail != null) {
	
			    		SecondaryLocators secondaryLocators = new SecondaryLocators();
			    		secondaryLocators.getSecondaryLocator().addAll(oracleCloneSpec.getStorageSpecification().getStorageMapping().
			    				getSecondaryConfigurationDatafiles().getSecondaryLocators().getSecondaryLocator());
			    		for (SecondaryLocator secLocator : secondaryLocators.getSecondaryLocator()) {
			    			found = false;
			    			CLIUtils.writeToLog("Primary Svm:" + secLocator.getPrimary().getSvm() + " Volume:" + secLocator.getPrimary().getVolume(), logger, LogLevel.INFO);
			    			CLIUtils.writeToLog("Secondary Svm:" + secLocator.getSecondary().getSvm() + " Volume:" + secLocator.getSecondary().getVolume(), logger, LogLevel.INFO);
				            for (SmArchivedDetail archivedDetail : arrayOfSmArchivedDetail.getSmArchivedDetail()) {
				            	SnapshotLocator primaryFromResponse = archivedDetail.getPrimary();
				                boolean foundSecondaryLocator = false;
				                CLIUtils.writeToLog(MessageBundle.localizeMessage(
				                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_MATCHING_PRIMARY_SVM_AND_VOLUME,
				                		secLocator.getPrimary().getSvm(), secLocator.getPrimary().getVolume(), 
				                		primaryFromResponse.getStorageSystemName(), primaryFromResponse.getStorageVolumeName()),
				                		logger, LogLevel.INFO);
				                
				                // If primary svm or volume doesnt match no need to check for secondary
				        		if (!secLocator.getPrimary().getSvm().equalsIgnoreCase(primaryFromResponse.getStorageSystemName()) ||
			        					!secLocator.getPrimary().getVolume().equalsIgnoreCase(primaryFromResponse.getStorageVolumeName()))
			        				continue;
	
				            	List<SnapshotLocator> snapshotLocators = archivedDetail.getSecondaries().getSnapshotLocator();
				            	for (SnapshotLocator secondaryFromResponse : snapshotLocators) {
				            		CLIUtils.writeToLog(MessageBundle.localizeMessage(
					                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_MATCHING_SECONDARY_SVM_AND_VOLUME,
					                		secLocator.getSecondary().getSvm(), secLocator.getSecondary().getVolume(), 
					                		secondaryFromResponse.getStorageSystemName(), secondaryFromResponse.getStorageVolumeName()),
					                		logger, LogLevel.INFO);
				        			if (secLocator.getSecondary().getSvm().equalsIgnoreCase(secondaryFromResponse.getStorageSystemName()) &&
				        					secLocator.getSecondary().getVolume().equalsIgnoreCase(secondaryFromResponse.getStorageVolumeName())){
				        				CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
						                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_BACKUP_REACHED_SECONDARY,backupName),
						                		logger, LogLevel.INFO);
				        				found = true;
				        				break;
				        			}
				        		}
				            }
				            if (!found)
				            	break;
			    		}
			    	}
	        	}
	        	catch(Exception e){
	        		CLIUtils.writeToLog(e.getMessage(), logger, LogLevel.INFO);
	        	}
	        	finally{
			    	if (!found){
			    		CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
		                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_WAITING_FOR_BACKUP,retryCount, waitTime),
		                		logger, LogLevel.INFO);
		            	Thread.sleep(waitTime * 1000);
		            	retryCount++;
			    	} else
			    		break;
	        	}
	        }
	        if (!found)
	        	throw new CLIException(MessageBundle.localizeMessage(
	        			CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_BACKUP_NOTFOUND_ONSECONDARY, backupName));
	    } catch (Exception e) {
	        throw new CLIException(e);
	    } finally {
	        // Stop Progress Bar
	        ProgressBarUtil.stopProgressBar(progressBar);
	    }
        CLIUtils.writeToLog("pollIfBackupReachedSecondary: Exit", logger,LogLevel.INFO);
    }
    
    private void pollIfLogBackupReachedSecondary(OracleCloneSpecification oracleCloneSpec, SmsBackupValidationHelper smsBackupValidationHelper, String logBackupId, String logBackupName) throws CLIException {

    	CLIUtils.writeToLog("pollIfLogBackupReachedSecondary: Enter", logger,LogLevel.INFO);
    	ProgressBar progressBar = new ProgressBar();
        try {	
        	int retryCount = 1;
	        int waitTime = 60;
	        long maxRetryCount = 15;
	        boolean found = false;
	        
	        if (secondaryUpdateWaitTimeout > 0) {
	        	maxRetryCount = secondaryUpdateWaitTimeout;
	        	CLIUtils.writeToLog("Setting max retry count to " + maxRetryCount, logger, LogLevel.INFO);
	        }
	        
	        while(retryCount <= maxRetryCount) {
	        	try{
	        		
			    	SmGetArchivedDetailsResponse smGetArchivedDetailsResponse = smsBackupValidationHelper
			                .getSecondaryBackupDetails(logBackupId, oracleCloneSpec.getAppobjectid(), appPluginCode, true);
			    	
			    	ArrayOfSmArchivedDetail arrayOfSmArchivedDetail = smGetArchivedDetailsResponse.getArchivedDetails();
	
			    	if (arrayOfSmArchivedDetail != null) {
	
			    		SecondaryLocators secondaryLocators = new SecondaryLocators();
			    		secondaryLocators.getSecondaryLocator().addAll(oracleCloneSpec.getStorageSpecification().getStorageMapping().
			    				getSecondaryConfigurationArchivelogs().getSecondaryLocators().getSecondaryLocator());
			    		for (SecondaryLocator secLocator : secondaryLocators.getSecondaryLocator()) {
			    			found = false;
			    			CLIUtils.writeToLog("Primary Svm:" + secLocator.getPrimary().getSvm() + " Volume:" + secLocator.getPrimary().getVolume(), logger, LogLevel.INFO);
			    			CLIUtils.writeToLog("Secondary Svm:" + secLocator.getSecondary().getSvm() + " Volume:" + secLocator.getSecondary().getVolume(), logger, LogLevel.INFO);
				            for (SmArchivedDetail archivedDetail : arrayOfSmArchivedDetail.getSmArchivedDetail()) {
				            	SnapshotLocator primaryFromResponse = archivedDetail.getPrimary();
				                boolean foundSecondaryLocator = false;
				                CLIUtils.writeToLog(MessageBundle.localizeMessage(
				                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_MATCHING_PRIMARY_SVM_AND_VOLUME,
				                		secLocator.getPrimary().getSvm(), secLocator.getPrimary().getVolume(), 
				                		primaryFromResponse.getStorageSystemName(), primaryFromResponse.getStorageVolumeName()),
				                		logger, LogLevel.INFO);
				                
				                // If primary svm or volume doesnt match no need to check for secondary
				        		if (!secLocator.getPrimary().getSvm().equalsIgnoreCase(primaryFromResponse.getStorageSystemName()) ||
			        					!secLocator.getPrimary().getVolume().equalsIgnoreCase(primaryFromResponse.getStorageVolumeName()))
			        				continue;
	
				            	List<SnapshotLocator> snapshotLocators = archivedDetail.getSecondaries().getSnapshotLocator();
				            	for (SnapshotLocator secondaryFromResponse : snapshotLocators) {
				            		CLIUtils.writeToLog(MessageBundle.localizeMessage(
					                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_MATCHING_SECONDARY_SVM_AND_VOLUME,
					                		secLocator.getSecondary().getSvm(), secLocator.getSecondary().getVolume(), 
					                		secondaryFromResponse.getStorageSystemName(), secondaryFromResponse.getStorageVolumeName()),
					                		logger, LogLevel.INFO);
				        			if (secLocator.getSecondary().getSvm().equalsIgnoreCase(secondaryFromResponse.getStorageSystemName()) &&
				        					secLocator.getSecondary().getVolume().equalsIgnoreCase(secondaryFromResponse.getStorageVolumeName())){
				        				CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
						                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_BACKUP_REACHED_SECONDARY,logBackupName),
						                		logger, LogLevel.INFO);
				        				found = true;
				        				break;
				        			}
				        		}
				            }
				            if (!found)
				            	break;
			    		}
			    	}
	        	}
	        	catch(Exception e){
	        		CLIUtils.writeToLog(e.getMessage(), logger, LogLevel.INFO);
	        	}
	        	finally{
			    	if (!found){
			    		CLIUtils.writeToLogAndConsole(MessageBundle.localizeMessage(
		                		CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_WAITING_FOR_LOG_BACKUP,retryCount, waitTime),
		                		logger, LogLevel.INFO);
		            	Thread.sleep(waitTime * 1000);
		            	retryCount++;
			    	} else
			    		break;
	        	}
	        }
	        if (!found)
	        	throw new CLIException(MessageBundle.localizeMessage(
	        			CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_BACKUP_NOTFOUND_ONSECONDARY, backupName));
	    } catch (Exception e) {
	        throw new CLIException(e);
	    } finally {
	        // Stop Progress Bar
	        ProgressBarUtil.stopProgressBar(progressBar);
	    }
        CLIUtils.writeToLog("pollIfLogBackupReachedSecondary: Exit", logger,LogLevel.INFO);
    }
    
    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse)
            throws CLIException {
        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public PluginCode getAppPluginCode() {
        return appPluginCode;
    }

    public void setAppPluginCode(PluginCode appPluginCode) {
        this.appPluginCode = appPluginCode;
    }

    public String getCloneToHost() {
        return cloneToHost;
    }

    public void setCloneToHost(String cloneToHost) {
        this.cloneToHost = cloneToHost;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public SMOCloneParameters getSmoCloneParameters() {
        return smoCloneParameters;
    }

    public void setSmoCloneParameters(SMOCloneParameters smoCloneParameters) {
        this.smoCloneParameters = smoCloneParameters;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupName() {
        return backupName;
    }
    
    public void setBackupId(String backupId){
    	this.backupId = backupId;
    }
    
    public String getBackupId() {
        return backupId;
    }

    public PrePostScriptParameters getPrePostScriptParameters() {
        return prePostScriptParameters;
    }

    public void setPrePostScriptParameters(
            PrePostScriptParameters prePostScriptParameters) {
        this.prePostScriptParameters = prePostScriptParameters;
    }

    public NotificationParameters getNotificationParameters() {
        return notificationParameters;
    }

    public void setNotificationParameters(NotificationParameters notificationParameters) {
        this.notificationParameters = notificationParameters;
    }

}
