/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/UnmountSmBackupCommand.java#4 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.unmountsmbackup.command.description")
public class UnmountSmBackupCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_UNMOUNTSMBACKUP_COMMAND_NAME);

    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.unmountsmbackup.command.backupName.option.description", required = true)
    private String backupName;

    @Parameter(names = { "-AppPluginCode" }, descriptionKey = "cli.unmountsmbackup.command.appPluginCode.option.description")
    private PluginCode appPluginCode = PluginCode.SCO;

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.unmountsmbackup.command.waitForCompletion.option.description")
    private boolean waitForCompletion;

    private boolean fileSystemMount = true;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        switch (appPluginCode) {
            case SCO:
                break;
            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        appPluginCode.name()));
        }

        if (StringUtils.isBlank(backupName)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_UNMOUNTSMBACKUP_BACKUP_NAME_EMPTY_ERROR));
        }
        CLIUtils.promptUserToContinue(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_UNMOUNTSMBACKUP_PROMPT_USER_UNMOUNT_CLONE));
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmDeleteCloneRequest, SmDeleteCloneResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmDeleteCloneRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmDeleteCloneResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_DELETE_CLONE_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse)
            throws CLIException {

        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public PluginCode getAppPluginCode() {
        return appPluginCode;
    }

    public void setAppPluginCode(PluginCode appPluginCode) {
        this.appPluginCode = appPluginCode;
    }

    public boolean isFileSystemMount() {
        return fileSystemMount;
    }

    public void setFileSystemMount(boolean fileSystemMount) {
        this.fileSystemMount = fileSystemMount;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupName() {
        return backupName;
    }
}