/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/output/formatter/invokesmresourceclonesplit/OptionCommonOutputFormatter.java#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.output.formatter.invokesmresourceclonesplit;

import org.apache.commons.lang.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCloneResourceSplitResponse;
import com.netapp.snapcenter.cli.command.invokesmcloneresourcesplit.CloneSplitCommandSubType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

/**
 * 
 * This class Prints the output for start option of Invoke-SmResourceSplit in
 * Text, Table, XML and JSON format
 *
 */

public class OptionCommonOutputFormatter implements OutputFormatter {

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(OptionCommonOutputFormatter.class);

    CloneSplitCommandSubType cloneSplitCommandSubType;

    public OptionCommonOutputFormatter(
            CloneSplitCommandSubType cloneSplitCommandSubType) {
        this.cloneSplitCommandSubType = cloneSplitCommandSubType;
    }

    @Override
    public <T> void formatAndPrintCommandResponseObject(T smResponse,
            FormatterType formatterType) throws CLIException {
        SmCloneResourceSplitResponse smCloneResourceSplitResponse = (SmCloneResourceSplitResponse) smResponse;

        if (StringUtils.isNotBlank(smCloneResourceSplitResponse.getResult()
                .getMessage())) {
            String errorMsg = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_RESPONSE_ERROR,
                            smCloneResourceSplitResponse.getResult()
                                    .getMessage());
            throw new CLIException(errorMsg);
        }

        switch (formatterType) {
            case TEXT:
                printOutputInTextFormat(smCloneResourceSplitResponse);
                break;
            case JSON:
            case XML:
                Formatter formatter = FormatterFactory.getFormatter(formatterType);
                formatter.writeFormattedObject(smCloneResourceSplitResponse);
                break;
            case TEXT_TABLE:
            default:
                printOutputInTableFormat(smCloneResourceSplitResponse);
        }
    }

    /**
     * Prints the output in table format.
     * 
     * @param smCloneResourceSplitEstimateResponse
     */
    @Override
    public void printOutputInTableFormat(
            SmCloneResourceSplitResponse smCloneResourceSplitResponse) {


        String headerJobId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TABLE_JOB_ID_HEADER);
        String headerAppObjectId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TABLE_APP_OBJECT_ID_HEADER);
        String headerCloneName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TABLE_CLONE_NAME_HEADER);

        String[] resourcesTableHeaders = { headerJobId, headerAppObjectId,
                headerCloneName };

        TextTableInterface resourcesTable = new TextTable(resourcesTableHeaders);

        String columnJobId = Long.toString(smCloneResourceSplitResponse
                .getJob().getId());
        String columnAppObjectId = smCloneResourceSplitResponse
                .getAppObjectId();
        String columnCloneName = smCloneResourceSplitResponse.getCloneName();

        resourcesTable.addRow(columnJobId, columnAppObjectId, columnCloneName);

        resourcesTable.displayFormattedTable();

    }

    /**
     * 
     * Prints the output in text format.
     * 
     * @param smCloneResourceSplitEstimateResponse
     */
    @Override
    public void printOutputInTextFormat(
            SmCloneResourceSplitResponse smCloneResourceSplitResponse) {

        StringBuilder commandOutput = new StringBuilder();

        String jobId = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TEXTVIEW_JOB_ID,
                        Long.toString(smCloneResourceSplitResponse.getJob()
                                .getId()));
        commandOutput.append(jobId);
        commandOutput.append(CLIUtils.NEW_LINE);

        String appObjecId = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TEXTVIEW_APP_OBJECT_ID,
                        smCloneResourceSplitResponse.getAppObjectId());
        commandOutput.append(appObjecId);
        commandOutput.append(CLIUtils.NEW_LINE);

        String cloneName = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMON_TEXTVIEW_CLONE_NAME,
                        smCloneResourceSplitResponse.getCloneName());
        commandOutput.append(cloneName);
        commandOutput.append(CLIUtils.NEW_LINE);
        commandOutput.append(CLIUtils.NEW_LINE);

        CLIUtils.writeToConsole(commandOutput.toString());
    }
}