/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parameters/BackupResourceGroupParameters.java.java $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parameters;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.builder.ToStringBuilder;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.contracts.ProtectionGroupType;

@Parameters(resourceBundle = "snapcenter-cli")
public class BackupResourceGroupParameters implements ParameterInterface {

    public static final String PARAMETER_CUSTOM_SNAPSHOT = "CustomSnapShotFormat";
    public static final String PARAMETER_CUSTOM_TEXT = "CustomText";
    public static final int CUSTOM_TEXT_MAX_LENGTH = 100;
    public static final String CUSTOM_SNAPSHOT_DATASET_FORMAT = "$ResourceGroup";
    public static final String CUSTOM_SNAPSHOT_POLICY_FORMAT = "$Policy";
    public static final String CUSTOM_SNAPSHOT_SCHEDULETYPE_FORMAT = "$ScheduleType";
    public static final String CUSTOM_SNAPSHOT_HOSTNAME_FORMAT = "$HostName";
    public static final String CUSTOM_SNAPSHOT_CUSTOMTEXT_FORMAT = "$CustomText";

    @Parameter(names = { "-Policies" }, descriptionKey = "cli.addsmresourcegroup.command.policies.option.description", splitter = CommaParameterSplitter.class)
    private List<String> policies = new ArrayList<>();

    @Parameter(names = { "-CustomSnapShotFormat" }, descriptionKey = "cli.addsmresourcegroup.command.customSnapShotFormat.option.description")
    private String customSnapshotFormat;

    @Parameter(names = { "-CustomText" }, descriptionKey = "cli.addsmresourcegroup.command.customText.option.description")
    private String customText;

    private ProtectionGroupType smProtectionGroupType = ProtectionGroupType.BACKUP;

    @Parameter(names = { "-SchedulerRunAsName" }, descriptionKey = "cli.addsmresourcegroup.command.schedulerRunAsName.option.description")
    private String schedulerRunAsName;

    @ParametersDelegate
    private NotificationParameters notificationParameters = new NotificationParameters();

    @Parameter(names = { "-CustomSnapshot" }, descriptionKey = "cli.addsmresourcegroup.command.customSnapShot.option.description")
    private boolean customSnapshot;

    @Parameter(names = { "-SecondaryLocator" }, descriptionKey = "cli.addsmresourcegroup.command.secondaryLocator.option.description", converter = CustomStringToMapConverter.class, splitter = NoParameterSplitter.class)
    private List<Map<String, String>> secondaryLocators;

    public List<String> getPolicies() {
        return policies;
    }

    public void setPolicies(List<String> policies) {
        this.policies = policies;
    }

    public String getCustomSnapshotFormat() {
        return customSnapshotFormat;
    }

    public void setCustomSnapshotFormat(String customSnapshotFormat) {
        this.customSnapshotFormat = customSnapshotFormat;
    }

    public String getCustomText() {
        return customText;
    }

    public void setCustomText(String customText) {
        this.customText = customText;
    }

    public ProtectionGroupType getSmProtectionGroupType() {
        return smProtectionGroupType;
    }

    public void setSmProtectionGroupType(
            ProtectionGroupType smProtectionGroupType) {
        this.smProtectionGroupType = smProtectionGroupType;
    }

    public String getSchedulerRunAsName() {
        return schedulerRunAsName;
    }

    public void setSchedulerRunAsName(String schedulerRunAsName) {
        this.schedulerRunAsName = schedulerRunAsName;
    }

    public NotificationParameters getNotificationParameters() {
        return notificationParameters;
    }

    public void setNotificationParameters(
            NotificationParameters notificationParameters) {
        this.notificationParameters = notificationParameters;
    }

    public void setCustomSnapshot(boolean customSnapshot) {
        this.customSnapshot = customSnapshot;
    }

    public boolean isCustomSnapshot() {
        return customSnapshot;
    }

    public List<Map<String, String>> getSecondaryLocators() {
        return secondaryLocators;
    }

    public void setSecondaryLocators(List<Map<String, String>> secondaryLocators) {
        this.secondaryLocators = secondaryLocators;
    }

    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this);
    }
}
