/*
 * $Id: //depot/prod/orion/sc_main/mintaka/spl/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parser/CommandParserImpl.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parser;

import com.beust.jcommander.JCommander;
import com.beust.jcommander.MissingCommandException;
import com.beust.jcommander.ParameterException;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.AddSmPolicyCommand;
import com.netapp.snapcenter.cli.command.AddSmProtectResourceCommand;
import com.netapp.snapcenter.cli.command.AddSmResourceGroupCommand;
import com.netapp.snapcenter.cli.command.AddSmRunAsCommand;
import com.netapp.snapcenter.cli.command.AddSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.CancelSmJobCommand;
import com.netapp.snapcenter.cli.command.CatalogSmBackupWithOracleRMANCommand;
import com.netapp.snapcenter.cli.command.CloseSmConnectionCommand;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.command.ConfigureSmOracleDatabaseCommand;
import com.netapp.snapcenter.cli.command.CopySmPolicyCommand;
import com.netapp.snapcenter.cli.command.CreateSmOracleCloneSpecificationCommand;
import com.netapp.snapcenter.cli.command.GetSmBackupCommand;
import com.netapp.snapcenter.cli.command.GetSmBackupDetailsCommand;
import com.netapp.snapcenter.cli.command.GetSmBackupReportCommand;
import com.netapp.snapcenter.cli.command.GetSmCloneCommand;
import com.netapp.snapcenter.cli.command.GetSmConfigSettingsCommand;
import com.netapp.snapcenter.cli.command.GetSmResourceGroupCommand;
import com.netapp.snapcenter.cli.command.GetSmHostCommand;
import com.netapp.snapcenter.cli.command.GetSmJobSummaryReportCommand;
import com.netapp.snapcenter.cli.command.GetSmLogsCommand;
import com.netapp.snapcenter.cli.command.GetSmOracleDatabaseConfigurationCommand;
import com.netapp.snapcenter.cli.command.GetSmPolicyCommand;
import com.netapp.snapcenter.cli.command.GetSmResourcesCommand;
import com.netapp.snapcenter.cli.command.GetSmRunAsCommand;
import com.netapp.snapcenter.cli.command.GetSmSecondaryDetailsCommand;
import com.netapp.snapcenter.cli.command.GetSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.GetSmSMTPServerCommand;
import com.netapp.snapcenter.cli.command.HelpCommand;
import com.netapp.snapcenter.cli.command.InvokeSmCloneResourceSplitCommand;
import com.netapp.snapcenter.cli.command.ModifySmPolicyCommand;
import com.netapp.snapcenter.cli.command.ModifySmResourceGroupCommand;
import com.netapp.snapcenter.cli.command.MountSmBackupCommand;
import com.netapp.snapcenter.cli.command.NewSmBackupCommand;
import com.netapp.snapcenter.cli.command.NewSmCloneCommand;
import com.netapp.snapcenter.cli.command.OpenSmConnectionCommand;
import com.netapp.snapcenter.cli.command.RefreshSmCloneCommand;
import com.netapp.snapcenter.cli.command.RemoveSmBackupCommand;
import com.netapp.snapcenter.cli.command.RemoveSmCloneCommand;
import com.netapp.snapcenter.cli.command.RemoveSmDataSetCommand;
import com.netapp.snapcenter.cli.command.RemoveSmProtectResourceCommand;
import com.netapp.snapcenter.cli.command.RemoveSmPolicyCommand;
import com.netapp.snapcenter.cli.command.RemoveSmRunAsCommand;
import com.netapp.snapcenter.cli.command.RemoveSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.RenameSmBackupCommand;
import com.netapp.snapcenter.cli.command.RestoreSmBackupCommand;
import com.netapp.snapcenter.cli.command.SetSmConfigSettingsCommand;
import com.netapp.snapcenter.cli.command.SetSmSMTPServerCommand;
import com.netapp.snapcenter.cli.command.ModifySmProtectResourceCommand;
import com.netapp.snapcenter.cli.command.SetSmRunAsCommand;
import com.netapp.snapcenter.cli.command.SetSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.UncatalogSmBackupWithOracleRMANCommand;
import com.netapp.snapcenter.cli.command.UnmountSmBackupCommand;
import com.netapp.snapcenter.cli.command.storage.preferences.AddSvmPreferredDataPathCommand;
import com.netapp.snapcenter.cli.command.storage.preferences.GetPreferredStorageExportIpsCommand;
import com.netapp.snapcenter.cli.command.storage.preferences.GetSvmPreferredDataPathCommand;
import com.netapp.snapcenter.cli.command.storage.preferences.RemoveSvmPreferredDataPathCommand;
import com.netapp.snapcenter.cli.command.storage.preferences.SetPreferredStorageExportIpsCommand;
import com.netapp.snapcenter.cli.command.storage.preferences.SetSvmPreferredDataPathCommand;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.startup.CLILauncher;
import com.netapp.snapcenter.cli.startup.ManPageInvocator;
import com.netapp.snapcenter.cli.utils.CLIUtils;

import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * JCommander based command line parser implementation
 */
public class CommandParserImpl implements CommandParserInterface {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CLILauncher.class);

    private Map<String, Class<?>> commandNameToClassMap;

    private static final String ADDSMRUNAS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMRUNAS_COMMAND_NAME);

    private static final String ADDSMRESOURCEGROUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMRESOURCEGROUP_COMMAND_NAME);

    /*
     * Masking Add-SmHost and Remvoe-SmHost Command since only remote install is supported from
     * GUI or PS private static final String ADDSMHOST_COMMAND_NAME =
     * MessageBundle.localizeMessage(CLIMsgKey.CLI_ADDSMHOST_COMMAND_NAME);
     * private static final String REMOVESMHOST_COMMAND_NAME = MessageBundle
     * .localizeMessage(CLIMsgKey.CLI_REMOVESMHOST_COMMAND_NAME);
     */

    private static final String ADDSMPOLICY_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMPOLICY_COMMAND_NAME);

    private static final String ADDSMSTORAGECONNECTION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMSTORAGECONNECTION_COMMAND_NAME);

    private static final String CATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME);

    private static final String CLOSESMCONNECTION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CLOSESMCONNECTION_COMMAND_NAME);

    private static final String CREATESMORACLECLONESPECIFICATION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_COMMAND_NAME);

    private static final String INVOKESMCLONERESOURCESPLIT_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_INVOKESMCLONERESOURCESPLIT_COMMAND_NAME);

    private static final String GETSMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_COMMAND_NAME);

    private static final String GETSMSECONDARYDETAILS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMSECONDARYDETAILS_COMMAND_NAME);

    private static final String GETSMBACKUPDETAILS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPDETAILS_COMMAND_NAME);

    private static final String GETSMCLONE_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_COMMAND_NAME);

    private static final String GETSMDATASET_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMDATASET_COMMAND_NAME);

    private static final String GETSMORACLEDATABASECONFIGURATION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMORACLEDATABASECONFIGURATION_COMMAND_NAME);

    private static final String GETSMLOGS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMLOGS_COMMAND_NAME);

    private static final String CONFIGURESMORACLEDATABASE_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_CONFIGURESMORACLEDATABASE_COMMAND_NAME);

    private static final String MODIFYSMRESOURCEGROUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_MODIFYSMRESOURCEGROUP_COMMAND_NAME);

    private static final String GETSMHOST_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMHOST_COMMAND_NAME);

    private static final String GETSMJOBSUMMARYREPORT_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMJOBSUMMARYREPORT_COMMAND_NAME);

    private static final String GETSMBACKUPREPORT_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPREPORT_COMMAND_NAME);

    private static final String GETSMPOLICY_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_COMMAND_NAME);

    private static final String GETSMRESOURCES_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMRESOURCES_COMMAND_NAME);

    private static final String GETSMRUNAS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMRUNAS_COMMAND_NAME);

    private static final String GETSMSTORAGECONNECTION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_COMMAND_NAME);

    private static final String MODIFYSMPOLICY_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_MODIFYSMPOLICY_COMMAND_NAME);

    private static final String MOUNTSMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_MOUNTSMBACKUP_COMMAND_NAME);

    private static final String NEWSMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_NEWSMBACKUP_COMMAND_NAME);

    private static final String NEWSMCLONE_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_COMMAND_NAME);

    private static final String OPENSMCONNECTION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_OPENSMCONNECTION_COMMAND_NAME);

    private static final String REMOVESMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMBACKUP_COMMAND_NAME);

    private static final String REMOVESMCLONE_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMCLONE_COMMAND_NAME);

    private static final String REMOVESMDATASET_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMDATASET_COMMAND_NAME);

    private static final String REMOVESMPROTECTRESOURCE_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMPROTECTRESOURCE_COMMAND_NAME);
    
    private static final String REMOVESMPOLICY_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMPOLICY_COMMAND_NAME);
    
    private static final String COPYSMPOLICY_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_COPYSMPOLICY_COMMAND_NAME);
    
    private static final String RENAMESMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_RENAMESMBACKUP_COMMAND_NAME);

    private static final String REMOVESMRUNAS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMRUNAS_COMMAND_NAME);

    private static final String REMOVESMSTORAGECONNECTION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMSTORAGECONNECTION_COMMAND_NAME);

    private static final String RESTORESMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_RESTORESMBACKUP_COMMAND_NAME);

    private static final String SETSMRUNAS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMRUNAS_COMMAND_NAME);

    private static final String SETSMSTORAGECONNECTION_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMSTORAGECONNECTION_COMMAND_NAME);

    private static final String UNMOUNTSMBACKUP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_UNMOUNTSMBACKUP_COMMAND_NAME);

    private static final String HELP_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_HELP_COMMAND_NAME);

    private static final String ADDSVMPREFERREDDATAPATH_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSVMPREFERREDDATAPATH_COMMAND_NAME);

    private static final String GETSVMPREFERREDDATAPATH_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSVMPREFERREDDATAPATH_COMMAND_NAME);

    private static final String SETSVMPREFERREDDATAPATH_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSVMPREFERREDDATAPATH_COMMAND_NAME);

    private static final String REMOVESVMPREFERREDDATAPATH_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESVMPREFERREDDATAPATH_COMMAND_NAME);

    private static final String SETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME);

    private static final String GETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME);


    private static final String UNCATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_UNCATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME);


    private static final String SETCONFIGSETTINGS_COMMAND_NAME = MessageBundle.localizeMessage(
            CLIMsgKey.CLI_SETSMCONFIGSETTINGS_COMMAND_NAME);
    private static final String GETCONFIGSETIINGS_COMMAND_NAME = MessageBundle.localizeMessage(
            CLIMsgKey.CLI_GETSMCONFIGSETTINGS_COMMAND_NAME);

    private static final String ADDSMPROTECTRESOURCE_COMMAND_NAME = MessageBundle.localizeMessage(
            CLIMsgKey.CLI_ADDSMPROTECTRESOURCE_COMMAND_NAME);

    private static final String SETSMPROTECTRESOURCE_COMMAND_NAME = MessageBundle.localizeMessage(
            CLIMsgKey.CLI_SETSMPROTECTRESOURCE_COMMAND_NAME);

    private static final String CANCELSMJOB_COMMAND_NAME = MessageBundle.localizeMessage(
		CLIMsgKey.CLI_CANCELSMJOB_COMMAND_NAME);

    private static final String GETSMSMTPSERVER_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMSMTPSERVER_COMMAND_NAME);

    private static final String SETSMSMTPSERVER_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMSMTPSERVER_COMMAND_NAME);
    
    private static final String REFRESHSMCLONE_COMMAND_NAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REFRESHSMCLONE_COMMAND_NAME);

    /**
     * Will store all commands in a tree map which will have command name to
     * class mapping.
     * 
     */
    public void registerAllCommands() {

        commandNameToClassMap = new TreeMap<>(String.CASE_INSENSITIVE_ORDER);

        commandNameToClassMap.put(ADDSMRUNAS_COMMAND_NAME,
                AddSmRunAsCommand.class);

        commandNameToClassMap.put(ADDSMRESOURCEGROUP_COMMAND_NAME,
                AddSmResourceGroupCommand.class);

        /*
         * Masking add-smhost command.
         * commandNameToClassMap.put(ADDSMHOST_COMMAND_NAME,
         * AddSmHostCommand.class);
         * commandNameToClassMap.put(REMOVESMHOST_COMMAND_NAME,
         * RemoveSmHostCommand.class);
         */

        commandNameToClassMap.put(ADDSMPOLICY_COMMAND_NAME,
                AddSmPolicyCommand.class);

        commandNameToClassMap.put(ADDSMSTORAGECONNECTION_COMMAND_NAME,
                AddSmStorageConnectionCommand.class);

        commandNameToClassMap.put(CATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME,
                CatalogSmBackupWithOracleRMANCommand.class);

        commandNameToClassMap.put(CLOSESMCONNECTION_COMMAND_NAME,
                CloseSmConnectionCommand.class);

        commandNameToClassMap.put(CONFIGURESMORACLEDATABASE_COMMAND_NAME,
                ConfigureSmOracleDatabaseCommand.class);

        commandNameToClassMap.put(CREATESMORACLECLONESPECIFICATION_COMMAND_NAME,
                CreateSmOracleCloneSpecificationCommand.class);

        commandNameToClassMap.put(INVOKESMCLONERESOURCESPLIT_COMMAND_NAME,
                InvokeSmCloneResourceSplitCommand.class);

        commandNameToClassMap.put(GETSMBACKUP_COMMAND_NAME,
                GetSmBackupCommand.class);

        commandNameToClassMap.put(GETSMSECONDARYDETAILS_COMMAND_NAME,
                GetSmSecondaryDetailsCommand.class);

        commandNameToClassMap.put(GETSMBACKUPDETAILS_COMMAND_NAME,
                GetSmBackupDetailsCommand.class);

        commandNameToClassMap.put(GETSMCLONE_COMMAND_NAME,
                GetSmCloneCommand.class);

        commandNameToClassMap.put(GETSMDATASET_COMMAND_NAME,
                GetSmResourceGroupCommand.class);

        commandNameToClassMap.put(GETSMLOGS_COMMAND_NAME,
                GetSmLogsCommand.class);

        commandNameToClassMap.put(GETSMORACLEDATABASECONFIGURATION_COMMAND_NAME,
                GetSmOracleDatabaseConfigurationCommand.class);

        commandNameToClassMap.put(MODIFYSMRESOURCEGROUP_COMMAND_NAME,
                ModifySmResourceGroupCommand.class);

        commandNameToClassMap.put(GETSMHOST_COMMAND_NAME,
                GetSmHostCommand.class);

        commandNameToClassMap.put(GETSMJOBSUMMARYREPORT_COMMAND_NAME,
                GetSmJobSummaryReportCommand.class);

        commandNameToClassMap.put(GETSMBACKUPREPORT_COMMAND_NAME,
                GetSmBackupReportCommand.class);

        commandNameToClassMap.put(GETSMPOLICY_COMMAND_NAME,
                GetSmPolicyCommand.class);

        commandNameToClassMap.put(GETSMRESOURCES_COMMAND_NAME,
                GetSmResourcesCommand.class);

        commandNameToClassMap.put(GETSMRUNAS_COMMAND_NAME,
                GetSmRunAsCommand.class);

        commandNameToClassMap.put(GETSMSTORAGECONNECTION_COMMAND_NAME,
                GetSmStorageConnectionCommand.class);

        commandNameToClassMap.put(MODIFYSMPOLICY_COMMAND_NAME,
                ModifySmPolicyCommand.class);

        commandNameToClassMap.put(MOUNTSMBACKUP_COMMAND_NAME,
                MountSmBackupCommand.class);

        commandNameToClassMap.put(NEWSMBACKUP_COMMAND_NAME,
                NewSmBackupCommand.class);

        commandNameToClassMap.put(NEWSMCLONE_COMMAND_NAME,
                NewSmCloneCommand.class);

        commandNameToClassMap.put(OPENSMCONNECTION_COMMAND_NAME,
                OpenSmConnectionCommand.class);
        
        commandNameToClassMap.put(REFRESHSMCLONE_COMMAND_NAME,
        		RefreshSmCloneCommand.class);

        commandNameToClassMap.put(REMOVESMBACKUP_COMMAND_NAME,
                RemoveSmBackupCommand.class);

        commandNameToClassMap.put(REMOVESMCLONE_COMMAND_NAME,
                RemoveSmCloneCommand.class);

        commandNameToClassMap.put(REMOVESMDATASET_COMMAND_NAME,
                RemoveSmDataSetCommand.class);
  
        commandNameToClassMap.put(REMOVESMPROTECTRESOURCE_COMMAND_NAME,
        		RemoveSmProtectResourceCommand.class);

        commandNameToClassMap.put(REMOVESMPOLICY_COMMAND_NAME,
                RemoveSmPolicyCommand.class);
        
        commandNameToClassMap.put(COPYSMPOLICY_COMMAND_NAME,
                CopySmPolicyCommand.class);
        
        commandNameToClassMap.put(RENAMESMBACKUP_COMMAND_NAME,
                RenameSmBackupCommand.class);

        commandNameToClassMap.put(REMOVESMRUNAS_COMMAND_NAME,
                RemoveSmRunAsCommand.class);

        commandNameToClassMap.put(REMOVESMSTORAGECONNECTION_COMMAND_NAME,
                RemoveSmStorageConnectionCommand.class);

        commandNameToClassMap.put(RESTORESMBACKUP_COMMAND_NAME,
                RestoreSmBackupCommand.class);

        commandNameToClassMap.put(SETSMRUNAS_COMMAND_NAME,
                SetSmRunAsCommand.class);

        commandNameToClassMap.put(SETSMSTORAGECONNECTION_COMMAND_NAME,
                SetSmStorageConnectionCommand.class);

        commandNameToClassMap.put(UNMOUNTSMBACKUP_COMMAND_NAME,
                UnmountSmBackupCommand.class);

        commandNameToClassMap.put(HELP_COMMAND_NAME, HelpCommand.class);

        commandNameToClassMap.put(ADDSVMPREFERREDDATAPATH_COMMAND_NAME,
                AddSvmPreferredDataPathCommand.class);

        commandNameToClassMap.put(GETSVMPREFERREDDATAPATH_COMMAND_NAME,
                GetSvmPreferredDataPathCommand.class);

        commandNameToClassMap.put(SETSVMPREFERREDDATAPATH_COMMAND_NAME,
                SetSvmPreferredDataPathCommand.class);

        commandNameToClassMap.put(REMOVESVMPREFERREDDATAPATH_COMMAND_NAME,
                RemoveSvmPreferredDataPathCommand.class);

        commandNameToClassMap.put(SETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME,
                SetPreferredStorageExportIpsCommand.class);


        commandNameToClassMap.put(SETCONFIGSETTINGS_COMMAND_NAME,
                SetSmConfigSettingsCommand.class);

        commandNameToClassMap.put(GETCONFIGSETIINGS_COMMAND_NAME,
                GetSmConfigSettingsCommand.class);

        commandNameToClassMap.put(ADDSMPROTECTRESOURCE_COMMAND_NAME,
                AddSmProtectResourceCommand.class);

        commandNameToClassMap.put(SETSMPROTECTRESOURCE_COMMAND_NAME,
                ModifySmProtectResourceCommand.class);
        commandNameToClassMap.put(CANCELSMJOB_COMMAND_NAME,
        		CancelSmJobCommand.class);
        commandNameToClassMap.put(REFRESHSMCLONE_COMMAND_NAME,
        		RefreshSmCloneCommand.class);

        /*

        TODO delete when log settings is completed

        commandNameToClassMap.put(SETLOGSETTINGS_COMMAND_NAME,
                SetSmLogSettingsCommand.class);*/

        commandNameToClassMap.put(GETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME,
                GetPreferredStorageExportIpsCommand.class);

        commandNameToClassMap.put(UNCATALOGSMBACKUPWITHORACLERMAN_COMMAND_NAME,
                UncatalogSmBackupWithOracleRMANCommand.class);

        commandNameToClassMap.put(GETSMSMTPSERVER_COMMAND_NAME,
                GetSmSMTPServerCommand.class);

        commandNameToClassMap.put(SETSMSMTPSERVER_COMMAND_NAME,
                SetSmSMTPServerCommand.class);
    }

    /**
     * This will fetch all registered commands
     */
    @Override
    public List<String> getAllRegisteredCommands() {
        List<String> commands = new ArrayList<String>();
        commands.addAll(commandNameToClassMap.keySet());
        return commands;
    }

    /**
     * Parses command line arguments
     * 
     * @param args
     *            Command line arguments
     * @return Parsed command object
     * @throws CLIException
     */
    @Override
    public CommandInterface parseCommand(String[] args) throws CLIException {

        return parseInvokedCommand(args);
    }

    /**
     * Parse the currently invoked command
     * 
     * @param args
     *            Command line arguments
     * @return Parsed command object
     * @throws CLIException
     */
    private CommandInterface parseInvokedCommand(String[] args)
            throws CLIException {

        CommandInterface commandObj = null;
        String commandName = null;
        try {
            /* Fetch the command name and check if it is a registered command */
            commandName = args[0].trim();
            if (!isRegisteredCommand(commandName)) {
                throw new MissingCommandException(
                        MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_INVALID_COMMAND, commandName,
                                CLILauncher.CLI_EXECUTABLE_NAME));
            }

            // Create a command object for respective command name
            commandObj = createCommandObjectFromCommandName(commandName);

            JCommander jc = new JCommander(commandObj);
            // This will make options case insensitive
            jc.setCaseSensitiveOptions(false);
            // Parse the options provided excluding command name
            jc.parse(Arrays.copyOfRange(args, 1, args.length));

        } catch (MissingCommandException e) {
            throw new CLIException(e.getLocalizedMessage(), e);
        } catch (ParameterException e) {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append(e.getLocalizedMessage());
            stringBuilder.append(CLIUtils.NEW_LINE);
            /**
             * If parsed command is one of registered command then print usage.
             */
            String parsedCommand = commandName;
            if (parsedCommand != null && isRegisteredCommand(parsedCommand)
                    && !parsedCommand.equalsIgnoreCase(HELP_COMMAND_NAME)) {
                stringBuilder.append(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_HELP_USAGE,
                        CLILauncher.CLI_EXECUTABLE_NAME,
                        getCaseSensitiveRegisteredCommadName(parsedCommand)));
            } else {
                stringBuilder.append(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_USAGE, CLILauncher.CLI_EXECUTABLE_NAME));
            }
            CLIException cliException = new CLIException(
                    stringBuilder.toString());
            throw cliException;
        }
        return commandObj;
    }

    /*
     * Fetches the case sensitive command name. example: input: add-smhost
     * output: Add-SmHost
     */
    public String getCaseSensitiveRegisteredCommadName(String parsedCommand) {
        for (String caseSensitiveCommandName : commandNameToClassMap.keySet()) {
            if (caseSensitiveCommandName.equalsIgnoreCase(parsedCommand)) {
                return caseSensitiveCommandName;
            }
        }
        return StringUtils.EMPTY;
    }

    /* Instantiate a command object based on the command name */
    private CommandInterface createCommandObjectFromCommandName(
            String commandName) throws CLIException {
        CommandInterface commandObj = null;
        try {
            commandObj = (CommandInterface) commandNameToClassMap.get(
                    commandName).newInstance();
        } catch (InstantiationException | IllegalAccessException e) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_INTERNAL_UNEXPECTED_ERROR),
                    e);
        }
        return commandObj;
    }

    @Override
    public void printUsage(String commandName) throws CLIException {
        StringBuilder stringBuilder = new StringBuilder();
        if (commandName.equalsIgnoreCase(HELP_COMMAND_NAME)) {
            CLIUtils.writeToConsole(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_HELP_COMMAND_USAGE,
                    CLILauncher.CLI_EXECUTABLE_NAME));
        } else if (isRegisteredCommand(commandName)) {
            showManPage(getCaseSensitiveRegisteredCommadName(commandName));
/*            commandName = getCaseSensitiveRegisteredCommadName(commandName);
            // TODO: Usage Format has to be enhanced.
            JCommander jCommander = new JCommander();
            jCommander.addCommand(commandName,
                    createCommandObjectFromCommandName(commandName));
            jCommander.usage(commandName, stringBuilder);
            return stringBuilder.toString();
*/        } else {
            String exceptionMessage = MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_INVALID_COMMAND, commandName,
                    CLILauncher.CLI_EXECUTABLE_NAME);
            throw new CLIException(exceptionMessage);
        }
    }

    private boolean isRegisteredCommand(String commandName) {
        if (commandNameToClassMap.containsKey(commandName)) {
            return true;
        }
        return false;
    }

    @Override
    public String getCommandDescription(String commandName) throws CLIException {
        JCommander jCommander = new JCommander();
        jCommander.addCommand(commandName,
                createCommandObjectFromCommandName(commandName));
        return jCommander.getCommandDescription(commandName);
    }

    private void showManPage(String commandName) {

        ManPageInvocator manPageInvocator = new ManPageInvocator(commandName);
        Thread manThread = new Thread(manPageInvocator);
        manThread.start();
        try {
            manThread.join();
        } catch (InterruptedException e) {
            logger.debug(
                    "Man Page Invocation Interruption Error: "
                            + e.getLocalizedMessage(), e);
        }
    }

}
