/*
 * $Id: //depot/prod/orion/sc_main/mintaka/spl/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parser/CustomStringToHostResourcesMapConverter.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.IStringConverter;
import com.beust.jcommander.ParameterException;
import com.beust.jcommander.converters.BaseConverter;
import com.netapp.common.logging.MessageBundle;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

public class CustomStringToHostResourcesMapConverter extends
        BaseConverter<Map<String, String>> implements
        IStringConverter<Map<String, String>> {

    public CustomStringToHostResourcesMapConverter(String optionName) {
        super(optionName);
    }

    public static final String ORACLE_DATABASE = "Oracle Database";
    public static final String RESOURCE_TYPE = "type";
    public static final String RESOURCE_NAMES = "names";
    public static final String RESOURCE_HOST = "host";
    public static final String RESOURCE_NAMES_DELIMETER = ";";

    @Override
    public Map<String, String> convert(String keyValueMapString) {
        Map<String, String> convertedMap = new HashMap<String, String>();

        if (StringUtils.isBlank(keyValueMapString)) {
            throw new ParameterException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK, getOptionName()));
        }

        String keyValueMapStringWithoutFlowerBraces = StringUtils.substringBetween(
                keyValueMapString, "{", "}");
        keyValueMapString = (keyValueMapStringWithoutFlowerBraces == null) ? keyValueMapString
                : keyValueMapStringWithoutFlowerBraces;

        String keyValueMapStringWithoutList = processResourceNamesList(keyValueMapString);

        String[] keyValuePairsArray = keyValueMapStringWithoutList.split(",");
        // We need minimum 3 key value pairs for resource
        if (keyValuePairsArray.length < 3) {
            throw new IllegalArgumentException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_INVALID_FORMAT));
        }
        for (int j = 0; j < keyValuePairsArray.length; j++) {
            String keyValuePair = keyValuePairsArray[j];
            String[] keyValue = keyValuePair.split("=");
            if (keyValue.length != 2) {
                throw new ParameterException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_OPTION_VALUE_TO_KEY_VALUE_FORMAT_CONVERSION_FAILED,
                                        getOptionName(), keyValuePair));
            }
            /*
             * Keys like host, type, names all are in lower case and hence to
             * provide flexibility for user making it case insensitive by
             * converting to lower case. Change this if there is a key which
             * should have fixed case or camel case kind.
             */
            String name = keyValue[0].trim().toLowerCase();
            String val = keyValue[1].trim();
            convertedMap.put(name, val);
        }

        validateResource(convertedMap);
        return convertedMap;
    }

    /*
     * TODO: Works Only for one pair of list inside []. Should Enhance it in
     * future for multiple lists.
     */
    /*
     * Extract the list from the input, change the delimiter and put it back without braces [].
     * 
     * Input contains: names=[db1,db2]
     * After processing: names=db1;db2
     * 
     */
    private String processResourceNamesList(String keyValueMapString) {
        // Extract the list objects inside [] first
        String listValuesInsideMap = StringUtils.substringBetween(
                keyValueMapString, "[", "]");

        // Validate resource names
        if (StringUtils.isBlank(listValuesInsideMap)) {
            throw new IllegalArgumentException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_INVALID_FORMAT));
        }
        for (String res : listValuesInsideMap.trim().split(",")) {
            if (StringUtils.isBlank(res)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_RESOURCES_NAME_CANNOT_BE_EMPTY,
                                        getOptionName()));
            }
        }

        // Change the delimiter for processing
        String listValuesInsideMapWithResourceNamesDelimiter = listValuesInsideMap
                .replaceAll(",", RESOURCE_NAMES_DELIMETER);

        // replace the list with processed names with delimiter
        return StringUtils.replace(keyValueMapString,
                String.format("[%s]", listValuesInsideMap),
                listValuesInsideMapWithResourceNamesDelimiter);
    }

    private static void validateResource(Map<String, String> resource) {
        List<Map<String, String>> resources = new ArrayList<>();
        resources.add(resource);
        validateResources(resources);
    }

    private static void validateResources(List<Map<String, String>> resources) {
        Set<String> resourceKeys = new HashSet<>();
        resourceKeys.add(RESOURCE_HOST);
        resourceKeys.add(RESOURCE_NAMES);
        resourceKeys.add(RESOURCE_TYPE);

        for (Map<String, String> resource : resources) {
            Set<String> keySet = resource.keySet();
            if (keySet.size() > resourceKeys.size()
                    || !keySet.containsAll(resourceKeys)) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_INVALID_FORMAT));
            }
        }
    }

}
