/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parser/CustomStringToMapConverter.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parser;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.IStringConverter;
import com.beust.jcommander.ParameterException;
import com.beust.jcommander.converters.BaseConverter;
import com.netapp.common.logging.MessageBundle;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

public class CustomStringToMapConverter extends
        BaseConverter<Map<String, String>> implements
        IStringConverter<Map<String, String>> {



    protected boolean nullValueAllowed;

    public CustomStringToMapConverter(String optionName) {
        super(optionName);
    }

    @Override
    public Map<String, String> convert(String keyValueMapString) {
        Map<String, String> keyValuePairsMap = new TreeMap<>(String.CASE_INSENSITIVE_ORDER);

        if (StringUtils.isBlank(keyValueMapString)) {
            throw new ParameterException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK, getOptionName()));
        }

        String keyValueMapStringWithoutFlowerBraces = StringUtils.substringBetween(
                keyValueMapString, "{", "}");
        keyValueMapString = (keyValueMapStringWithoutFlowerBraces == null) ? keyValueMapString
                : keyValueMapStringWithoutFlowerBraces;

        String[] keyValuePairsArray = keyValueMapString.split(",");
        for (int j = 0; j < keyValuePairsArray.length; j++) {
            String keyValuePair = keyValuePairsArray[j];
            String[] keyValue = keyValuePair.split("=");

            if(nullValueAllowed) {
                String key = new String();
                String value = new String();
                if(keyValue.length == 2){
                    key = StringUtils.trim(keyValue[0]);
                    value = StringUtils.trim(keyValue[1]);
                } else {
                    key = StringUtils.trim(keyValue[0]);
                    value = StringUtils.EMPTY;
                }

                keyValuePairsMap.put(key, value);
            } else {
                if (keyValue.length != 2 ) {
                    throw new ParameterException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_OPTION_VALUE_TO_KEY_VALUE_FORMAT_CONVERSION_FAILED,
                                            getOptionName(), keyValuePair));
                }
                String key = StringUtils.trim(keyValue[0]);
                String val = StringUtils.trim(keyValue[1]);
                keyValuePairsMap.put(key, val);
            }
        }
        return keyValuePairsMap;
    }

    public boolean isNullValueAllowed() {
        return nullValueAllowed;
    }

    public void setNullValueAllowed(boolean nullValueAllowed) {
        this.nullValueAllowed = nullValueAllowed;
    }

}