/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parser/CustomStringToPrePostScriptMapConverter.java#1 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parser;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.ParameterException;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

public class CustomStringToPrePostScriptMapConverter extends
        CustomStringToSchedulesMapConverter {

    public static final String PRE_SCRIPT_PATH = "preScriptPath";
    public static final String PRE_SCRIPT_ARGUMENTS = "preScriptArguments";
    public static final String POST_SCRIPT_PATH = "postScriptPath";
    public static final String POST_SCRIPT_ARGUMENTS = "postScriptArguments";
    public static final String SCRIPT_TIMEOUT = "scriptTimeout";

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CustomStringToPrePostScriptMapConverter.class);

    public CustomStringToPrePostScriptMapConverter(String optionName) {
        super(optionName);
    }

    @Override
    public Map<String, String> convert(String keyValueMapString) {
        Map<String, String> convertedMap = new HashMap<String, String>();

        if (StringUtils.isBlank(keyValueMapString)) {
            throw new ParameterException(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK, getOptionName()));
        }

        String keyValueMapStringWithoutFlowerBraces = StringUtils
                .substringBetween(keyValueMapString, "{", "}");
        keyValueMapString = (keyValueMapStringWithoutFlowerBraces == null) ? keyValueMapString
                : keyValueMapStringWithoutFlowerBraces;

        String[] keyValuePairsArray = keyValueMapString.split(",");

        for (int j = 0; j < keyValuePairsArray.length; j++) {
            String keyValuePair = keyValuePairsArray[j];
            String[] keyValue = keyValuePair.split("=");
            if (keyValue.length != 2) {
                throw new ParameterException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_OPTION_VALUE_TO_KEY_VALUE_FORMAT_CONVERSION_FAILED,
                                        getOptionName(), keyValuePair));
            }
            /*
             * Keys like host, type, names all are in lower case and hence to
             * provide flexibility for user making it case insensitive by
             * converting to lower case. Change this if there is a key which
             * should have fixed case or camel case kind.
             */
            String name = keyValue[0].trim().toLowerCase();
            String val = keyValue[1].trim();
            convertedMap.put(name, val);
        }

        validateOptions(convertedMap);
        CLIUtils.writeToLog("Processing verification PrePostScript parameters"
                + convertedMap.keySet(), logger, LogLevel.DEBUG);
        return convertedMap;
    }

    private static void validateOptions(Map<String, String> resources) {
        Set<String> resourceKeys = new HashSet<>();

        resourceKeys.add(PRE_SCRIPT_PATH.toLowerCase());
        resourceKeys.add(PRE_SCRIPT_ARGUMENTS.toLowerCase());
        resourceKeys.add(POST_SCRIPT_PATH.toLowerCase());
        resourceKeys.add(POST_SCRIPT_ARGUMENTS.toLowerCase());
        resourceKeys.add(SCRIPT_TIMEOUT.toLowerCase());

        if ((resources.keySet().size() > resourceKeys.size())
                || (!resourceKeys.containsAll(resources.keySet()))) {

            throw new IllegalArgumentException(
                    MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_VERIFICATION_PRE_POST_SCRIPT_PARAMETERS_INVALID_COMMAND_ARGUMENTS));

        }
    }

}
