/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parser/CustomStringToSchedulesMapConverter.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.IStringConverter;
import com.beust.jcommander.ParameterException;
import com.beust.jcommander.converters.BaseConverter;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

public class CustomStringToSchedulesMapConverter extends BaseConverter<Map<String, String>>
        implements IStringConverter<Map<String, String>> {

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CustomStringToSchedulesMapConverter.class);

    public CustomStringToSchedulesMapConverter(String optionName) {
        super(optionName);
    }

    public static final String POLICY_NAME = "policyname";
    public static final String SCHEDULE_TYPE = "scheduletype";
    public static final String START_DATE_TIME = "starttime";
    public static final String END_DATE_TIME = "endtime";
    public static final String REPEATE_EVERY_HOUR = "repeat_every_hour";
    public static final String OCCURS_EVERY_MINUTE = "occurs_every_minute";
    public static final String OCCURS_EVERY_MONTH = "occurs_every_month";
    public static final String DAYS_INTERVAL = "daysinterval";
    public static final String DAYS_OF_THE_WEEK = "daysoftheweek";
    public static final String MONTHS_OF_THE_YEAR = "monthsoftheyear";
    public static final String DAYS_OF_THE_MONTH = "daysofthemonth";
    public static final String SCHEDULE_OPTIONS_LIST_DELIMETER = ";";

    @Override
    public Map<String, String> convert(String keyValueMapString) {
        Map<String, String> convertedMap = new HashMap<String, String>();

        if (StringUtils.isBlank(keyValueMapString)) {
            throw new ParameterException(
                    MessageBundle.localizeMessage(CLIMsgKey.CLI_PARAMETER_CANNOT_BE_BLANK, getOptionName()));
        }

        String keyValueMapStringWithoutFlowerBraces = StringUtils.substringBetween(keyValueMapString, "{", "}");
        keyValueMapString = (keyValueMapStringWithoutFlowerBraces == null) ? keyValueMapString
                : keyValueMapStringWithoutFlowerBraces;

        String keyValueMapStringWithoutList = processAllLists(keyValueMapString);

        String[] keyValuePairsArray = keyValueMapStringWithoutList.split(",");

        for (int j = 0; j < keyValuePairsArray.length; j++) {
            String keyValuePair = keyValuePairsArray[j];
            String[] keyValue = keyValuePair.split("=");
//            CLIUtils.writeToLog(String.format("The following parameter is being processed '%s'", keyValue[0]), logger,
//                    LogLevel.DEBUG);
            if (keyValue.length != 2) {
                throw new ParameterException(
                        MessageBundle.localizeMessage(CLIMsgKey.CLI_OPTION_VALUE_TO_KEY_VALUE_FORMAT_CONVERSION_FAILED,
                                getOptionName(), keyValuePair));
            }
            /*
             * Keys like host, type, names all are in lower case and hence to
             * provide flexibility for user making it case insensitive by
             * converting to lower case. Change this if there is a key which
             * should have fixed case or camel case kind.
             */
            String name = keyValue[0].trim().toLowerCase();
            String val = keyValue[1].trim();
          //Check for duplicate arguments
            if (StringUtils.isNotEmpty(convertedMap.get(name))) {
                throw new ParameterException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_COMMAND_DUPLICATE_ARGUMENTS_PASSED_TO_COMMAND_OPTIONS,
                                        "Schedules"));
            }
            convertedMap.put(name, val);
        }
        validateOptions(convertedMap);
        return convertedMap;
    }

    /*
     * Extract the list from the input, change the delimiter and put it back
     * without braces [].
     * 
     * Input contains:
     * DaysOfTheWeek=[db1,db2],MonthsOfTheYear=[2,12],DaysOfTheMonth=[29,4]
     * After processing:DaysOfTheWeek=[db1;db2] MonthsOfTheYear=[2;12]
     * DaysOfTheMonth=[29;4]
     * 
     */
    private String processAllLists(String keyValueMapString) {
        // Extract the list objects inside [] first

        List<String> splitStringList = listKeyValueMapString(keyValueMapString);
        String modifiedKeyValueMapString = null;
        for (String splitString : splitStringList) {

            if (StringUtils.indexOf(splitString, "[") > 0) {
                // Extract the list objects inside [] first
                String listValuesInsideMap = StringUtils.substringBetween(splitString, "[", "]");
                // Validate resource names
                if (StringUtils.isBlank(listValuesInsideMap)) {
                    throw new IllegalArgumentException(
                            MessageBundle.localizeMessage(CLIMsgKey.CLI_SCHEDULES_MAPPING_EMPTY_VALUES,splitString));
                }
                for (String res : listValuesInsideMap.trim().split(",")) {
                    if (StringUtils.isBlank(res)) {
                        throw new IllegalArgumentException(MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_SCHEDULES_MAPPING_OPTIONS_CANNOT_BE_EMPTY, getOptionName()));
                    }
                }

                // Change the delimiter for processing
                String listValuesInsideMapWithResourceNamesDelimiter = listValuesInsideMap.replaceAll(",",
                        SCHEDULE_OPTIONS_LIST_DELIMETER);

                // replace the list with processed names with delimiter
                modifiedKeyValueMapString = StringUtils.replace(
                        (modifiedKeyValueMapString == null) ? keyValueMapString : modifiedKeyValueMapString,
                        String.format("[%s]", listValuesInsideMap), listValuesInsideMapWithResourceNamesDelimiter);
            }
        }
        return (modifiedKeyValueMapString == null) ? keyValueMapString : modifiedKeyValueMapString;
    }

    /**
     * This method sepraters the string if it has a "]," so that each of the
     * substring can be processed seperately Input
     * contains:PolicyName=FullBackup1,ScheduleType=Weekly,DaysOfTheWeek=[db1,
     * db2],MonthsOfTheYear=[2,12],DaysOfTheMonth=[29,4] Output contains:
     * "PolicyName=FullBackup1,ScheduleType=Weekly,DaysOfTheWeek=[db1,db2]",
     * "MonthsOfTheYear=[2,12]","DaysOfTheMonth=[29,4]"
     * 
     * @param keyValueMapString
     * @return
     */
    private List<String> listKeyValueMapString(String keyValueMapString) {
        int i = 0;
        String tempString = keyValueMapString;
        List<String> splitStringList = new ArrayList<String>();
        do {

            i = StringUtils.indexOf(tempString, "],");
            if (i > 0) {
                String listValuesInsideMap = StringUtils.substring(tempString, 0, i + 1);
                tempString = StringUtils.substring(tempString, i + 2, tempString.length());
                splitStringList.add(listValuesInsideMap);
            }
        } while (i > 0);
        if (!StringUtils.isBlank(tempString)) {
            splitStringList.add(tempString);
        }
        return splitStringList;
    }

    private static void validateOptions(Map<String, String> resource) {
        List<Map<String, String>> resources = new ArrayList<>();
        resources.add(resource);
        validateOptions(resources);
    }

    private static void validateOptions(List<Map<String, String>> resources) {
        Set<String> resourceKeys = new HashSet<>();
        resourceKeys.add(POLICY_NAME);
        resourceKeys.add(SCHEDULE_TYPE);
        resourceKeys.add(START_DATE_TIME);
        resourceKeys.add(END_DATE_TIME);
        resourceKeys.add(REPEATE_EVERY_HOUR);
        resourceKeys.add(OCCURS_EVERY_MINUTE);
        resourceKeys.add(OCCURS_EVERY_MONTH);
        resourceKeys.add(DAYS_INTERVAL);
        resourceKeys.add(MONTHS_OF_THE_YEAR);
        resourceKeys.add(DAYS_OF_THE_WEEK);
        resourceKeys.add(DAYS_OF_THE_MONTH);

        for (Map<String, String> resource : resources) {
            Set<String> keySet = resource.keySet();
            if ((keySet.size() > resourceKeys.size())
                    || (!resourceKeys.containsAll(keySet))) {
                throw new IllegalArgumentException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_SCHEDULES_MAPPING_INVALID_ARGUMENTS));
            }
        }
    }

}
