/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/storage/preferences/AbstractStoragePreferencesCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.storage.preferences;

import java.util.Map.Entry;
import java.util.Set;

import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.spl.common.propertiesfile.helpers.PropertiesFileOperationImpl;
import com.netapp.spl.common.propertiesfile.helpers.PropertiesFileOperationInterface;

public abstract class AbstractStoragePreferencesCommand {

    public static final String STORAGE_PREFERENCE_PROPERTIES_FILE_HEADER = String
            .format("%s\n%s\n%s\n%s",
                    " Storage Preference Configuration",
                    " The file has been created by NetApp",
                    " Do not directly edit/modify this file, use 'sccli' instead for configuration and management of datapaths",
                    " Last modified");
    public static final String STORAGE_PREFERENCE_PROPERTIES_FILE_PATH = "/var/opt/snapcenter/scu/etc/storagepreference.properties";
    public static final String PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY = "PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT";

    private PropertiesFileOperationInterface storagePreferencesProperties;

    protected void initializeStoragePreferencesProperties() throws CLIException {
        try {
            storagePreferencesProperties = new PropertiesFileOperationImpl(
                    STORAGE_PREFERENCE_PROPERTIES_FILE_PATH);
            storagePreferencesProperties.init();
        } catch (Exception e) {
            throw new CLIException(e.getLocalizedMessage(), e);
        }
    }

    protected String getStoragePreferencesProperty(String key) {
        return storagePreferencesProperties.getProperty(key);
    }

    protected Set<Entry<Object, Object>> getAllStoragePreferencesProperties() {
        return storagePreferencesProperties.getAllProperties();
    }

    protected void saveStoragePreferencesProperty(String key, String value)
            throws CLIException {
        try {
            storagePreferencesProperties.saveProperty(key, value,
                    STORAGE_PREFERENCE_PROPERTIES_FILE_HEADER);
        } catch (Exception e) {
            throw new CLIException(e.getLocalizedMessage(), e);
        }
    }

    protected void deleteStoragePreferencesProperty(String key)
            throws CLIException {
        try {
            storagePreferencesProperties.deleteProperty(key,
                    STORAGE_PREFERENCE_PROPERTIES_FILE_HEADER);
        } catch (Exception e) {
            throw new CLIException(e.getLocalizedMessage(), e);
        }
    }

}