/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/SmOracleDatabaseConfigurationHelper.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.helper;

import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_CONFIGURE_DATABASE_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_DISCOVER_RESOURCE_DETAILS_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_GET_RESOURCE_HOST_LIST_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_RBAC_REMOVE_OBJECT_AUTHENTICATION_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_RBAC_SET_OBJECT_AUTHENTICATION_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_RBAC_VALIDATE_OBJECT_CREDENTIAL_SERVICE_PATH;
import static com.netapp.spl.common.restclient.config.SMSRestConfiguration.SMS_SET_RESOURCE_PREFERRED_HOSTS_SERVICE_PATH;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.netapp.smcore.plugin.contracts.ArrayOfSmAuthInfo;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHost;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObject;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObjectOperationResult;
import com.netapp.smcore.plugin.contracts.ArrayOfSmString;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmAuthInfo;
import com.netapp.smcore.plugin.contracts.SmAuthMode;
import com.netapp.smcore.plugin.contracts.SmAuthRequest;
import com.netapp.smcore.plugin.contracts.SmAuthResponse;
import com.netapp.smcore.plugin.contracts.SmConfigureDatabaseRequest;
import com.netapp.smcore.plugin.contracts.SmGetResourceDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetResourceDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmGetResourceHostListRequest;
import com.netapp.smcore.plugin.contracts.SmGetResourceHostListResponse;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SmSetResourcePreferredHostsRequest;
import com.netapp.smcore.plugin.contracts.SmString;
import com.netapp.smcore.plugin.contracts.SmValidateObjectCredentialRequest;
import com.netapp.smcore.plugin.contracts.SmValidateObjectCredentialResponse;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

/**
 * This helper class contains snapcenter backup service calls. It provides the
 * API's for validating backups.
 *
 */
public class SmOracleDatabaseConfigurationHelper {

    public static final int DATABASE_OR_ASM_DEFAULT_PORT = 1521;

    private String dbResourceId;
    private String dbName;
    private String hostOrClusterName;

    private SnapCenterServerServiceHelper snapCenterServerServiceHelper;

    public SmOracleDatabaseConfigurationHelper(
            SnapCenterServerServiceHelper serviceHelper, String dbResourceId) {
        this.snapCenterServerServiceHelper = serviceHelper;
        this.dbResourceId = dbResourceId;
        String[] resSplit = StringUtils.split(dbResourceId, "\\");
        this.hostOrClusterName = StringUtils.trim(resSplit[0]);
        this.dbName = StringUtils.trim(resSplit[1]);
    }

    public SmOracleDb getOracleDatabaseDetails() throws CLIException {
        SmGetResourceDetailsRequest smGetResourceDetailsRequest = new SmGetResourceDetailsRequest();
        SmOracleDb smOracleDb = new SmOracleDb();
        smOracleDb.setId(getDbResourceId());
        smGetResourceDetailsRequest.setAppObject(smOracleDb);
        SMSRestInvocationContext<SmGetResourceDetailsRequest, SmGetResourceDetailsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smGetResourceDetailsRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetResourceDetailsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_DISCOVER_RESOURCE_DETAILS_SERVICE_PATH);
        SmGetResourceDetailsResponse response = (SmGetResourceDetailsResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            if (response.getAppObject() instanceof SmOracleDb) {
                return (SmOracleDb) response.getAppObject();
            } else {
                return null;
            }
        } else {
            throw new CLIException(response.getResult().getMessage());
        }
    }

    public ArrayOfSmHost getOracleClusterDatabaseHostsList()
            throws CLIException {

        SmGetResourceHostListRequest smGetResourceHostListRequest = new SmGetResourceHostListRequest();
        SmOracleDb smOracleDb = new SmOracleDb();
        smOracleDb.setId(getDbResourceId());
        smGetResourceHostListRequest.setAppObject(smOracleDb);
        SMSRestInvocationContext<SmGetResourceHostListRequest, SmGetResourceHostListResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smGetResourceHostListRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetResourceHostListResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_GET_RESOURCE_HOST_LIST_SERVICE_PATH);
        SmGetResourceHostListResponse response = (SmGetResourceHostListResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            return response.getHosts();
        } else {
            throw new CLIException(response.getResult().getMessage());
        }

    }

    public ArrayOfSmObjectOperationResult validateAuth(Long dbRunAsId, int dbPort, Long asmRunAsId,
            int asmPort, Long rmanCatalogRunAsId, String tnsName) throws CLIException {
        SmValidateObjectCredentialRequest smValidateObjectCredentialRequest = new SmValidateObjectCredentialRequest();
        ArrayOfSmObject appObjs = new ArrayOfSmObject();

        SmOracleDb smOracleDb = new SmOracleDb();
        smOracleDb.setId(getDbResourceId());
        smOracleDb.setName(getDbName());
        smOracleDb.setDbSid(getDbName());
        smOracleDb.setHost(getHostOrClusterName());

        smOracleDb.setDatabasePort(dbPort);
        smOracleDb.setAsmPort(asmPort);

        appObjs.getSmObject().add(smOracleDb);
        smValidateObjectCredentialRequest.setAppObjects(appObjs);

        // Db Run as
        SmAuth smAuth = new SmAuth();
        SmAuthMode dbAuthMode = SmAuthMode.NONE;
        if (dbRunAsId != null) {
            dbAuthMode = SmAuthMode.ORACLE_DATABASE;
            smAuth.setId(dbRunAsId);
        }

        smAuth.setAuthMode(dbAuthMode);
        smValidateObjectCredentialRequest.setAuth(smAuth);

        // ASM Run as
        SmAuth asmAuth = new SmAuth();
        SmAuthMode asmAuthMode = SmAuthMode.NONE;
        if (asmRunAsId != null) {
            asmAuthMode = SmAuthMode.ORACLE_ASM;
            asmAuth.setId(asmRunAsId);
        }

        asmAuth.setAuthMode(asmAuthMode);
        smValidateObjectCredentialRequest.setAsmAuth(asmAuth);

        // RMAN Catalog Run as
        SmAuth rmanCatalogAuth = new SmAuth();
        SmAuthMode rmanCatalogAuthMode = SmAuthMode.NONE;
        if (rmanCatalogRunAsId != null) {
            rmanCatalogAuthMode = SmAuthMode.ORACLE_RMAN_CATALOG;
            rmanCatalogAuth.setId(rmanCatalogRunAsId);
        }

        rmanCatalogAuth.setAuthMode(rmanCatalogAuthMode);
        smValidateObjectCredentialRequest.setRmanCatalogAuth(rmanCatalogAuth);

        SMSRestInvocationContext<SmValidateObjectCredentialRequest, SmValidateObjectCredentialResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smValidateObjectCredentialRequest);
        smsRestInvocationContext
                .setResponseClass(SmValidateObjectCredentialResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_RBAC_VALIDATE_OBJECT_CREDENTIAL_SERVICE_PATH);
        SmValidateObjectCredentialResponse response = (SmValidateObjectCredentialResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            return response.getValidateObjectOperationResults();
        } else {
            throw new CLIException(response.getResult().getMessage());
        }
    }

    public void addAuth(Long runAsId, SmAuthMode smAuthMode)
            throws CLIException {

        SmAuthRequest smAuthRequest = new SmAuthRequest();
        smAuthRequest.setAppllySameConfiguration(true);

        SmAuthInfo authInfo = new SmAuthInfo();

        SmObject smObject = new SmObject();
        smObject.setId(getDbResourceId());
        authInfo.setAppObject(smObject);

        ArrayOfSmAuthInfo arrayOfSmAuthInfo = new ArrayOfSmAuthInfo();
        arrayOfSmAuthInfo.getSmAuthInfo().add(authInfo);
        smAuthRequest.setAuthInfos(arrayOfSmAuthInfo);

        SmAuth smAuth = new SmAuth();
        smAuth.setId(runAsId);
        smAuth.setAuthMode(smAuthMode);
        smAuthRequest.setAuth(smAuth);

        SMSRestInvocationContext<SmAuthRequest, SmAuthResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smAuthRequest);
        smsRestInvocationContext.setResponseClass(SmAuthResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_RBAC_SET_OBJECT_AUTHENTICATION_SERVICE_PATH);
        SmAuthResponse response = (SmAuthResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            return;
        } else {
            throw new CLIException(response.getResult().getMessage());
        }

    }

    public void removeAuth(Long runAsId, SmAuthMode smAuthMode)
            throws CLIException {

        SmAuthRequest smAuthRequest = new SmAuthRequest();
        smAuthRequest.setAppllySameConfiguration(true);

        SmAuthInfo authInfo = new SmAuthInfo();

        SmObject smObject = new SmObject();
        smObject.setId(getDbResourceId());
        authInfo.setAppObject(smObject);

        ArrayOfSmAuthInfo arrayOfSmAuthInfo = new ArrayOfSmAuthInfo();
        arrayOfSmAuthInfo.getSmAuthInfo().add(authInfo);
        smAuthRequest.setAuthInfos(arrayOfSmAuthInfo);

        SmAuth smAuth = new SmAuth();
        smAuth.setId(runAsId);
        smAuth.setAuthMode(smAuthMode);
        smAuthRequest.setAuth(smAuth);

        SMSRestInvocationContext<SmAuthRequest, SmAuthResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smAuthRequest);
        smsRestInvocationContext.setResponseClass(SmAuthResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_RBAC_REMOVE_OBJECT_AUTHENTICATION_SERVICE_PATH);
        SmAuthResponse response = (SmAuthResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            return;
        } else {
            throw new CLIException(response.getResult().getMessage());
        }

    }

    public void configurePortAndRmanCatalogTnsName(int dbPort, int asmPort, String rmanCatalogTnsName) throws CLIException {

        SmConfigureDatabaseRequest smConfigureDatabaseRequest = new SmConfigureDatabaseRequest();
        smConfigureDatabaseRequest.setResourceId(getDbResourceId());
        smConfigureDatabaseRequest.setPort(dbPort);
        smConfigureDatabaseRequest.setASMPort(asmPort);
        smConfigureDatabaseRequest.setRmanCatalogTnsName(rmanCatalogTnsName);

        SMSRestInvocationContext<SmConfigureDatabaseRequest, SmResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smConfigureDatabaseRequest);
        smsRestInvocationContext.setResponseClass(SmResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_CONFIGURE_DATABASE_SERVICE_PATH);
        SmResponse response = (SmResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            return;
        } else {
            throw new CLIException(response.getResult().getMessage());
        }

    }

    public void setPreferredNodes(List<String> preferredHosts)
            throws CLIException {

        SmSetResourcePreferredHostsRequest smSetResourcePreferredHostsRequest = new SmSetResourcePreferredHostsRequest();
        SmOracleDb smOracleDb = new SmOracleDb();
        smOracleDb.setId(getDbResourceId());
        smSetResourcePreferredHostsRequest.setAppObject(smOracleDb);
        ArrayOfSmString prefHosts = new ArrayOfSmString();
        for (String prefHost : preferredHosts) {
            SmString smString = new SmString();
            smString.setValue(prefHost);
            prefHosts.getSmString().add(smString);
        }
        smSetResourcePreferredHostsRequest.setPreferredHosts(prefHosts);
        // Invoke Server rest end point
        SMSRestInvocationContext<SmSetResourcePreferredHostsRequest, SmResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest(smSetResourcePreferredHostsRequest);
        smsRestInvocationContext.setResponseClass(SmResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMS_SET_RESOURCE_PREFERRED_HOSTS_SERVICE_PATH);
        SmResponse response = (SmResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            return;
        } else {
            throw new CLIException(response.getResult().getMessage());
        }

    }

    public boolean isOracleRacDatabase(SmOracleDb smOracleDb) {
        return StringUtils.isNotBlank(smOracleDb.getRacName());
    }

    public SnapCenterServerServiceHelper getSnapCenterServerServiceHelper() {
        return snapCenterServerServiceHelper;
    }

    public String getDbResourceId() {
        return dbResourceId;
    }

    public String getDbName() {
        return dbName;
    }

    public String getHostOrClusterName() {
        return hostOrClusterName;
    }

}
